# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import datetime
import sys
from typing import Any, Callable, Dict, IO, List, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._serialization import Serializer
from .._vendor import _format_url_section

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_assets_list_assets_request(
    *,
    external_idprefix: Optional[str] = None,
    name: Optional[str] = None,
    source: Optional[str] = None,
    limit: int = 100,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets"

    # Construct parameters
    if external_idprefix is not None:
        _params["externalIdprefix"] = _SERIALIZER.query("external_idprefix", external_idprefix, "str")
    if name is not None:
        _params["name"] = _SERIALIZER.query("name", name, "str")
    if source is not None:
        _params["source"] = _SERIALIZER.query("source", source, "str")
    if limit is not None:
        _params["limit"] = _SERIALIZER.query("limit", limit, "int")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_assets_create_asset_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_assets_search_assets_request(
    *, name: Optional[str] = None, description: Optional[str] = None, limit: int = 100, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets/search"

    # Construct parameters
    if name is not None:
        _params["name"] = _SERIALIZER.query("name", name, "str")
    if description is not None:
        _params["description"] = _SERIALIZER.query("description", description, "str")
    if limit is not None:
        _params["limit"] = _SERIALIZER.query("limit", limit, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_assets_get_asset_by_id_request(id: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_assets_update_asset_request(id: str, *, json: Optional[JSON] = None, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, json=json, **kwargs)


def build_assets_get_asset_structure_by_id_request(id: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets/{id}/structure"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_assets_list_machines_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/assets/machines"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_azure_regions_list_azure_regions_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/azureregions"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_cdf_get_cdf_config_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/admin/cdf/config"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_cdf_update_cdf_config_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/admin/cdf/config"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_countries_list_countries_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/countries"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_customer_orders_upsert_customer_order_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/customerorders/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_customer_orders_upsert_customer_order_line_request(id: str, line: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/customerorders/{id}/orderlines/{line}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "line": _SERIALIZER.url("line", line, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_customers_upsert_customer_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/customers/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_customers_delete_customer_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/erp/customers/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_customers_list_customers_request(
    *, page_size: int = 50, continuation_token_parameter: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/customers"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_workorders_upsert_workorder_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_list_workorders_request(
    *,
    page_size: int = 50,
    search: Optional[str] = None,
    active_orders: Optional[bool] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if search is not None:
        _params["search"] = _SERIALIZER.query("search", search, "str")
    if active_orders is not None:
        _params["activeOrders"] = _SERIALIZER.query("active_orders", active_orders, "bool")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_workorders_upsert_workorder_traces_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders/{id}/traces"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_set_workorder_customer_order_reference_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders/{id}/customerorderreference"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_check_resource_status_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/resources/{id}/status"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_workorders_start_workorder_operation_request(id: str, operation: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders/{id}/operations/{operation}/start"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_stop_workorder_operation_request(id: str, operation: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders/{id}/operations/{operation}/stop"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_register_workorder_operation_event_request(id: str, operation: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/{id}/operations/{operation}/events"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_start_workorder_operation_v2_request(id: str, operation: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/v2/erp/workorders/{id}/operations/{operation}/start"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_stop_workorder_operation_v2_request(id: str, operation: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/v2/erp/workorders/{id}/operations/{operation}/stop"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_workorders_list_active_workorder_operations_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/activeworkorderoperations"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_workorders_delete_workorder_operation_event_request(
    id: str, operation: int, event_id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/erp/workorders/{id}/operations/{operation}/events/{eventId}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "operation": _SERIALIZER.url("operation", operation, "int"),
        "eventId": _SERIALIZER.url("event_id", event_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_workorders_get_workorder_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_workorders_delete_workorder_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/erp/workorders/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_workorders_get_workorder_operation_event_request(event_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/operations/events/{eventId}"
    path_format_arguments = {
        "eventId": _SERIALIZER.url("event_id", event_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_workorders_update_workorder_operation_event_timestamps_request(event_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/erp/workorders/operations/events/{eventId}"
    path_format_arguments = {
        "eventId": _SERIALIZER.url("event_id", event_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PATCH", url=_url, headers=_headers, **kwargs)


def build_workorders_filter_workorder_operation_events_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/erp/workorders/operations/events/filter"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_external_list_invites_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/external/invites"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_external_accept_supplier_invite_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/external/invites"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_external_list_companies_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/external/companies"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_external_access_list_users_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/externalaccess/users"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_external_access_create_company_user_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/externalaccess/users"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_external_access_update_company_user_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/externalaccess/users/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_external_access_delete_company_user_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/externalaccess/users/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_external_access_list_roles_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/externalaccess/roles"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_external_access_list_company_customers_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/externalaccess/customers"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_external_access_delete_company_customer_request(tenant_id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/externalaccess/customers/{tenantId}"
    path_format_arguments = {
        "tenantId": _SERIALIZER.url("tenant_id", tenant_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_electrical_list_electrical_source_types_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/electrical/sourcetypes"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_electrical_list_electrical_data_configs_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/electrical"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_electrical_create_electrical_iot_config_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/electrical"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_electrical_delete_electrical_iot_config_request(type_id: str, id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/iot/electrical/{typeId}/{id}"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_welding_list_welding_source_types_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/welding/sourcetypes"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_welding_list_electrical_data_configs_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/welding"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_welding_create_welding_iot_config_request(*, json: Optional[JSON] = None, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/iot/welding"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, json=json, **kwargs)


def build_welding_delete_welding_iot_config_request(type_id: str, id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/iot/welding/{typeId}/{id}"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_links_get_all_links_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/links"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_links_create_global_link_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/links/global"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_machine_alarms_list_machine_alarms_request(
    *,
    asset_id: Optional[int] = None,
    start_time: Optional[datetime.datetime] = None,
    end_time: Optional[datetime.datetime] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machinealarms"

    # Construct parameters
    if asset_id is not None:
        _params["assetId"] = _SERIALIZER.query("asset_id", asset_id, "int")
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")
    if end_time is not None:
        _params["endTime"] = _SERIALIZER.query("end_time", end_time, "iso-8601")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_alarms_list_alarms_per_machine_request(
    *,
    start_time: Optional[datetime.datetime] = None,
    end_time: Optional[datetime.datetime] = None,
    asset_id: Optional[int] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machinealarms/alarmspermachine"

    # Construct parameters
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")
    if end_time is not None:
        _params["endTime"] = _SERIALIZER.query("end_time", end_time, "iso-8601")
    if asset_id is not None:
        _params["assetId"] = _SERIALIZER.query("asset_id", asset_id, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_alarms_list_alarm_severity_occurrences_request(
    *,
    start_time: Optional[datetime.datetime] = None,
    end_time: Optional[datetime.datetime] = None,
    asset_id: Optional[int] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machinealarms/alarmtypeseverities"

    # Construct parameters
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")
    if end_time is not None:
        _params["endTime"] = _SERIALIZER.query("end_time", end_time, "iso-8601")
    if asset_id is not None:
        _params["assetId"] = _SERIALIZER.query("asset_id", asset_id, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_alarms_get_machine_alarm_details_request(id: int, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machinealarms/alarms/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_machine_utilization_get_machine_utilizations_request(
    *,
    asset_id: Optional[int] = None,
    favorites: bool = False,
    start_time_today: Optional[datetime.datetime] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization"

    # Construct parameters
    if asset_id is not None:
        _params["assetId"] = _SERIALIZER.query("asset_id", asset_id, "int")
    if favorites is not None:
        _params["favorites"] = _SERIALIZER.query("favorites", favorites, "bool")
    if start_time_today is not None:
        _params["startTimeToday"] = _SERIALIZER.query("start_time_today", start_time_today, "iso-8601")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_utilization_get_utilization_data_point_for_machine_request(
    id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization/{id}/utilizationdatapoints"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_utilization_get_utilization_details_for_machine_request(
    id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization/{id}/utilization"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_utilization_list_n_days_power_on_utilization_datapoints_request(
    *, id: Optional[int] = None, external_id: Optional[str] = None, n_days: Optional[int] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization/power-on/datapoints"

    # Construct parameters
    if id is not None:
        _params["id"] = _SERIALIZER.query("id", id, "int")
    if external_id is not None:
        _params["externalId"] = _SERIALIZER.query("external_id", external_id, "str")
    if n_days is not None:
        _params["nDays"] = _SERIALIZER.query("n_days", n_days, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_machine_utilization_get_factory_utilization_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization/factory"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_machine_utilization_list_machine_states_request(
    id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/machineutilization/{id}/machine-states"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "int"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if start_time is not None:
        _params["startTime"] = _SERIALIZER.query("start_time", start_time, "iso-8601")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_me_get_my_apps_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/me/apps"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_me_get_current_user_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/me"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_list_measurment_form_schemas_request(
    *,
    page_size: int = 50,
    customer_id: Optional[str] = None,
    customer_name: Optional[str] = None,
    part_number: Optional[str] = None,
    part_revision: Optional[str] = None,
    drawing: Optional[str] = None,
    drawing_revision: Optional[str] = None,
    filter: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if customer_id is not None:
        _params["customerId"] = _SERIALIZER.query("customer_id", customer_id, "str")
    if customer_name is not None:
        _params["customerName"] = _SERIALIZER.query("customer_name", customer_name, "str")
    if part_number is not None:
        _params["partNumber"] = _SERIALIZER.query("part_number", part_number, "str")
    if part_revision is not None:
        _params["partRevision"] = _SERIALIZER.query("part_revision", part_revision, "str")
    if drawing is not None:
        _params["drawing"] = _SERIALIZER.query("drawing", drawing, "str")
    if drawing_revision is not None:
        _params["drawingRevision"] = _SERIALIZER.query("drawing_revision", drawing_revision, "str")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_measurement_form_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_schema_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_update_measurement_form_schema_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_update_schema_grouped_elements_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}/groupedelements"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_upload_schema_drawing_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}/uploaddrawing"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_upload_schema_attachment_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}/uploadattachment"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_import_status_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{id}/importstatus"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_schema_element_request(schema_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/elements"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_list_linkable_measurement_form_schemas_request(
    schema_id: str,
    *,
    page_size: int = 50,
    filter: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/listlinkableschemas"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_measurement_form_schema_link_request(
    schema_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/link"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_delete_measurement_form_schema_link_request(
    schema_id: str, linked_schema_id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/link/{linkedSchemaId}"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
        "linkedSchemaId": _SERIALIZER.url("linked_schema_id", linked_schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_form_schemas_release_schema_request(schema_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/release"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_revoke_schema_request(schema_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/revoke"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_draft_version_request(schema_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/draftversion"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_is_mapping_from_previous_version_required_request(
    schema_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/{schemaId}/mappingfrompreviousversionrequired"
    path_format_arguments = {
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_settings_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/settings"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_update_measurement_form_settings_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/settings"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_list_measurement_form_mappings_from_schema_request(
    *, target_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/mapping"

    # Construct parameters
    _params["targetId"] = _SERIALIZER.query("target_id", target_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_measurement_form_mapping_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/mapping"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_mapping_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/mapping/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_delete_measurement_form_mapping_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/mapping/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_form_schemas_set_measurement_form_mapping_balloons_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/mapping/{id}/balloons"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_mapping_suggestion_request(
    *, target_id: str, source_id: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/mappingsuggestion"

    # Construct parameters
    _params["targetId"] = _SERIALIZER.query("target_id", target_id, "str")
    if source_id is not None:
        _params["sourceId"] = _SERIALIZER.query("source_id", source_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_import_measurement_form_schema_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/import"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_list_measurement_form_needs_request(
    *,
    page_size: int = 50,
    customer_id: Optional[str] = None,
    customer_name: Optional[str] = None,
    part_number: Optional[str] = None,
    part_revision: Optional[str] = None,
    drawing: Optional[str] = None,
    drawing_revision: Optional[str] = None,
    filter: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    only_without_drawing_url: Optional[bool] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/needs"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if customer_id is not None:
        _params["customerId"] = _SERIALIZER.query("customer_id", customer_id, "str")
    if customer_name is not None:
        _params["customerName"] = _SERIALIZER.query("customer_name", customer_name, "str")
    if part_number is not None:
        _params["partNumber"] = _SERIALIZER.query("part_number", part_number, "str")
    if part_revision is not None:
        _params["partRevision"] = _SERIALIZER.query("part_revision", part_revision, "str")
    if drawing is not None:
        _params["drawing"] = _SERIALIZER.query("drawing", drawing, "str")
    if drawing_revision is not None:
        _params["drawingRevision"] = _SERIALIZER.query("drawing_revision", drawing_revision, "str")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )
    if only_without_drawing_url is not None:
        _params["onlyWithoutDrawingUrl"] = _SERIALIZER.query(
            "only_without_drawing_url", only_without_drawing_url, "bool"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_set_measurement_form_need_user_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measurementforms/schemas/needs/{id}/assign"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_set_measurement_form_need_as_not_needed_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/needs/{id}/notneeded"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_create_measurement_form_schema_from_need_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/needs/{id}/schema"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_upload_need_drawing_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/needs/{id}/uploaddrawing"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_list_measurment_form_schemas_not_needed_request(
    *,
    page_size: int = 50,
    customer_id: Optional[str] = None,
    customer_name: Optional[str] = None,
    part_number: Optional[str] = None,
    part_revision: Optional[str] = None,
    drawing: Optional[str] = None,
    drawing_revision: Optional[str] = None,
    filter: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/schemasnotneeded"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if customer_id is not None:
        _params["customerId"] = _SERIALIZER.query("customer_id", customer_id, "str")
    if customer_name is not None:
        _params["customerName"] = _SERIALIZER.query("customer_name", customer_name, "str")
    if part_number is not None:
        _params["partNumber"] = _SERIALIZER.query("part_number", part_number, "str")
    if part_revision is not None:
        _params["partRevision"] = _SERIALIZER.query("part_revision", part_revision, "str")
    if drawing is not None:
        _params["drawing"] = _SERIALIZER.query("drawing", drawing, "str")
    if drawing_revision is not None:
        _params["drawingRevision"] = _SERIALIZER.query("drawing_revision", drawing_revision, "str")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_set_measurment_form_schemas_not_needed_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/schemasnotneeded"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_delete_measurment_form_schemas_not_needed_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/schemasnotneeded/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_form_schemas_delete_imported_measurment_schemas_request(**kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/imported"

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_form_schemas_list_resource_types_for_needs_generator_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/resourcetypes"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_add_resource_type_for_needs_generator_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/resourcetypes"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_delete_resource_type_for_needs_generator_request(
    resource_type: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/resourcetypes/{resourceType}"
    path_format_arguments = {
        "resourceType": _SERIALIZER.url("resource_type", resource_type, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="GET", url=_url, **kwargs)


def build_measurement_form_schemas_list_frequencies_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/frequencies"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_get_measurement_form_instance_schema_feedback_request(
    *,
    page_size: int = 50,
    search: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/schemas/feedback"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if search is not None:
        _params["search"] = _SERIALIZER.query("search", search, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_form_schemas_set_measurement_form_instance_schema_feedback_user_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measurementforms/schemas/feedback/{id}/assign"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_form_schemas_delete_measurement_form_instance_schema_feedback_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/schemas/feedback/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_forms_instances_list_active_measurement_forms_request(
    *,
    page_size: int = 50,
    search: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    tenant_id: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if search is not None:
        _params["search"] = _SERIALIZER.query("search", search, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_list_measurement_forms_by_status_request(
    *,
    status: Optional[Union[str, "_models.MeasurementFormInstanceStatus"]] = None,
    page_size: int = 50,
    search: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/bystatus"

    # Construct parameters
    if status is not None:
        _params["status"] = _SERIALIZER.query("status", status, "str")
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if search is not None:
        _params["search"] = _SERIALIZER.query("search", search, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_get_measurement_form_instance_request(
    id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_create_measurement_form_instance_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_update_measurement_form_instance_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_complete_measurement_form_instance_request(
    id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/complete"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_reactivate_measurement_form_instance_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/reactivate"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_approve_measurement_form_instance_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/approve"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_disapprove_measurement_form_instance_request(
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/disapprove"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_cancel_measurement_form_instance_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/cancel"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_get_measurement_form_instance_schema_request(
    id: str, schema_id: str, *, sequence: Optional[str] = None, tenant_id: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/schemas/{schemaId}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if sequence is not None:
        _params["sequence"] = _SERIALIZER.query("sequence", sequence, "str")
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_get_workorder_measurement_form_progress_request(
    id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/progress"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_get_audit_log_request(
    id: str,
    *,
    tenant_id: Optional[str] = None,
    schema_id: Optional[str] = None,
    sequence: Optional[str] = None,
    element_id: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/auditlog"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")
    if schema_id is not None:
        _params["schemaId"] = _SERIALIZER.query("schema_id", schema_id, "str")
    if sequence is not None:
        _params["sequence"] = _SERIALIZER.query("sequence", sequence, "str")
    if element_id is not None:
        _params["elementId"] = _SERIALIZER.query("element_id", element_id, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_get_validation_rules_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/validationrules"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_set_cmm_element_request(
    id: str,
    *,
    schema_id: Optional[str] = None,
    element_id: Optional[str] = None,
    tenant_id: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/measurementforms/instances/{id}/setcmm"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if schema_id is not None:
        _params["schemaId"] = _SERIALIZER.query("schema_id", schema_id, "str")
    if element_id is not None:
        _params["elementId"] = _SERIALIZER.query("element_id", element_id, "str")
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    return HttpRequest(method="POST", url=_url, params=_params, **kwargs)


def build_measurement_forms_instances_remove_cmm_element_request(
    id: str,
    *,
    schema_id: Optional[str] = None,
    element_id: Optional[str] = None,
    tenant_id: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/measurementforms/instances/{id}/removecmm"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if schema_id is not None:
        _params["schemaId"] = _SERIALIZER.query("schema_id", schema_id, "str")
    if element_id is not None:
        _params["elementId"] = _SERIALIZER.query("element_id", element_id, "str")
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    return HttpRequest(method="POST", url=_url, params=_params, **kwargs)


def build_measurement_forms_instances_save_value_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/value"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_save_tool_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/tool"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_save_comment_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/comment"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_batch_insert_values_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/batchvalues"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_save_measurement_form_instance_schema_comment_request(
    id: str, schema_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measurementforms/instances/{id}/schemas/{schemaId}/comment"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_create_measurement_form_instance_schema_feedback_request(
    id: str, schema_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    tenant_id = kwargs.pop("tenant_id", _params.pop("tenantId", None))  # type: Optional[str]
    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/schemas/{schemaId}/feedback"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "schemaId": _SERIALIZER.url("schema_id", schema_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if tenant_id is not None:
        _params["tenantId"] = _SERIALIZER.query("tenant_id", tenant_id, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_measurement_forms_instances_list_supplier_measurement_form_instances_request(
    supplier_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/suppliers/{supplierId}"
    path_format_arguments = {
        "supplierId": _SERIALIZER.url("supplier_id", supplier_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_upsert_supplier_to_measurment_form_instance_request(
    id: str, supplier_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measurementforms/instances/{id}/suppliers/{supplierId}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "supplierId": _SERIALIZER.url("supplier_id", supplier_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measurement_forms_instances_remove_supplier_from_measurment_form_instance_request(
    id: str, supplier_id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/instances/{id}/suppliers/{supplierId}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "supplierId": _SERIALIZER.url("supplier_id", supplier_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_forms_instances_delete_all_measurement_schema_instances_request(**kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measurementforms/instances/all"

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measurement_forms_instances_export_dimension_report_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measurementforms/instances/{id}/report"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_list_measuring_tools_request(
    *,
    page: Optional[int] = None,
    page_size: int = 100,
    tool_id: Optional[str] = None,
    tool_name: Optional[str] = None,
    type_id: Optional[str] = None,
    sub_type_id: Optional[str] = None,
    order_by: Optional[str] = None,
    sort_direction: Optional[str] = None,
    include_deprecated: bool = False,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools"

    # Construct parameters
    if page is not None:
        _params["page"] = _SERIALIZER.query("page", page, "int")
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if tool_id is not None:
        _params["toolId"] = _SERIALIZER.query("tool_id", tool_id, "str")
    if tool_name is not None:
        _params["toolName"] = _SERIALIZER.query("tool_name", tool_name, "str")
    if type_id is not None:
        _params["typeId"] = _SERIALIZER.query("type_id", type_id, "str")
    if sub_type_id is not None:
        _params["subTypeId"] = _SERIALIZER.query("sub_type_id", sub_type_id, "str")
    if order_by is not None:
        _params["orderBy"] = _SERIALIZER.query("order_by", order_by, "str")
    if sort_direction is not None:
        _params["sortDirection"] = _SERIALIZER.query("sort_direction", sort_direction, "str")
    if include_deprecated is not None:
        _params["includeDeprecated"] = _SERIALIZER.query("include_deprecated", include_deprecated, "bool")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measuring_tools_create_measuring_tool_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_update_measuring_tool_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_get_measuring_tool_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_list_calibrations_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/{id}/calibrations"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_register_calibration_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/{id}/calibrations"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_attach_calibration_certificate_request(
    id: str, calibration_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/{id}/calibrations/{calibrationId}/attachcertificate"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "calibrationId": _SERIALIZER.url("calibration_id", calibration_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_delete_calibration_certificate_request(
    id: str, calibration_id: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/measuringtools/{id}/calibrations/{calibrationId}/deletecertificate"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
        "calibrationId": _SERIALIZER.url("calibration_id", calibration_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measuring_tools_list_manufacturers_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/manufacturers"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_create_manufacturer_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/manufacturers"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_list_measuring_tool_types_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_create_measuring_tool_type_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_update_measuring_tool_type_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_delete_measuring_tool_type_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measuringtools/types/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measuring_tools_list_measuring_tool_sub_types_request(type_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types/{typeId}/subtypes"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_create_measuring_tool_sub_type_request(type_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types/{typeId}/subtypes"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_update_measuring_tool_sub_type_request(type_id: str, id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/types/{typeId}/subtypes/{id}"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_delete_measuring_tool_sub_type_request(type_id: str, id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measuringtools/types/{typeId}/subtypes/{id}"
    path_format_arguments = {
        "typeId": _SERIALIZER.url("type_id", type_id, "str"),
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measuring_tools_list_measuring_units_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/units"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_create_measuring_unit_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/units"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_delete_measuring_unit_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/measuringtools/units/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_measuring_tools_get_measuring_tool_whitelist_request(
    *,
    page: Optional[int] = None,
    page_size: int = 100,
    filter: Optional[str] = None,
    order_by: Optional[str] = None,
    sort_direction: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/whitelist"

    # Construct parameters
    if page is not None:
        _params["page"] = _SERIALIZER.query("page", page, "int")
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if order_by is not None:
        _params["orderBy"] = _SERIALIZER.query("order_by", order_by, "str")
    if sort_direction is not None:
        _params["sortDirection"] = _SERIALIZER.query("sort_direction", sort_direction, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measuring_tools_whitelist_measuring_tool_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/whitelist"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_update_whitelisted_measuring_tool_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measuringtools/whitelist/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_remove_measuring_tool_from_whitelist_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/whitelist/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_list_unregistered_tool_values_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/unregistered"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_deprecate_tool_request(id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    # Construct URL
    _url = "/measuringtools/deprecated/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="PUT", url=_url, headers=_headers, **kwargs)


def build_measuring_tools_get_calibration_list_request(
    *, status: Optional[Union[str, "_models.CalibrationListStatus"]] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/calibrationlist"

    # Construct parameters
    if status is not None:
        _params["status"] = _SERIALIZER.query("status", status, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_measuring_tools_import_measuring_tool_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/measuringtools/importtool"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_power_list_power_regions_request(*, country: Optional[str] = None, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/power/regions"

    # Construct parameters
    if country is not None:
        _params["country"] = _SERIALIZER.query("country", country, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_presentation_get_component_settings_request(component_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/presentation/components/{componentId}"
    path_format_arguments = {
        "componentId": _SERIALIZER.url("component_id", component_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_suppliers_list_supplier_invites_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/suppliers/supplierinvites"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_suppliers_create_supplier_invite_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/suppliers/supplierinvites"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_suppliers_delete_supplier_invite_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/suppliers/supplierinvites/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_suppliers_list_suppliers_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/suppliers"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_suppliers_delete_supplier_request(id: str, **kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/suppliers/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_suppliers_import_supplier_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/suppliers/import"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_test_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/test"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_consumption_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_consumption_with_automatic_granularity_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption/autogranularity"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_power_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/power"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_customer_order_consumption_request(external_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption/customerorders/{externalId}"
    path_format_arguments = {
        "externalId": _SERIALIZER.url("external_id", external_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_work_order_consumption_request(external_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption/workorders/{externalId}"
    path_format_arguments = {
        "externalId": _SERIALIZER.url("external_id", external_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_get_product_consumption_request(external_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption/product/{externalId}"
    path_format_arguments = {
        "externalId": _SERIALIZER.url("external_id", external_id, "str"),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_sustainability_create_customer_order_line_ghg_report_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/sustainability/consumption/customerorderlinereport"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_upload_create_upload_info_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/upload"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_users_list_users_request(
    *,
    page_size: int = 50,
    filter: Optional[str] = None,
    continuation_token_parameter: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/users"

    # Construct parameters
    if page_size is not None:
        _params["pageSize"] = _SERIALIZER.query("page_size", page_size, "int")
    if filter is not None:
        _params["filter"] = _SERIALIZER.query("filter", filter, "str")
    if continuation_token_parameter is not None:
        _params["continuationToken"] = _SERIALIZER.query(
            "continuation_token_parameter", continuation_token_parameter, "str"
        )

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class AssetsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`assets` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_assets(
        self,
        *,
        external_idprefix: Optional[str] = None,
        name: Optional[str] = None,
        source: Optional[str] = None,
        limit: int = 100,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AssetDtoPagedResult:
        """list_assets.

        :keyword external_idprefix: Default value is None.
        :paramtype external_idprefix: str
        :keyword name: Default value is None.
        :paramtype name: str
        :keyword source: Default value is None.
        :paramtype source: str
        :keyword limit: Default value is 100.
        :paramtype limit: int
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: AssetDtoPagedResult
        :rtype: ~ignos.api.client.models.AssetDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.AssetDtoPagedResult]

        request = build_assets_list_assets_request(
            external_idprefix=external_idprefix,
            name=name,
            source=source,
            limit=limit,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("AssetDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_asset(
        self, body: Optional[_models.CreateAsset] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AssetDto:
        """create_asset.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateAsset
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AssetDto
        :rtype: ~ignos.api.client.models.AssetDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_asset(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AssetDto:
        """create_asset.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: AssetDto
        :rtype: ~ignos.api.client.models.AssetDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_asset(self, body: Optional[Union[_models.CreateAsset, IO]] = None, **kwargs: Any) -> _models.AssetDto:
        """create_asset.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateAsset or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: AssetDto
        :rtype: ~ignos.api.client.models.AssetDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.AssetDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateAsset")
            else:
                _json = None

        request = build_assets_create_asset_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("AssetDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def search_assets(
        self, *, name: Optional[str] = None, description: Optional[str] = None, limit: int = 100, **kwargs: Any
    ) -> List[_models.AssetDto]:
        """search_assets.

        :keyword name: Default value is None.
        :paramtype name: str
        :keyword description: Default value is None.
        :paramtype description: str
        :keyword limit: Default value is 100.
        :paramtype limit: int
        :return: list of AssetDto
        :rtype: list[~ignos.api.client.models.AssetDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.AssetDto]]

        request = build_assets_search_assets_request(
            name=name,
            description=description,
            limit=limit,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[AssetDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_asset_by_id(self, id: int, **kwargs: Any) -> _models.AssetDto:
        """get_asset_by_id.

        :param id: Required.
        :type id: int
        :return: AssetDto
        :rtype: ~ignos.api.client.models.AssetDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.AssetDto]

        request = build_assets_get_asset_by_id_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("AssetDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def update_asset(self, id: str, body: Optional[JSON] = None, **kwargs: Any) -> _models.AssetDto:
        """update_asset.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: JSON
        :return: AssetDto
        :rtype: ~ignos.api.client.models.AssetDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", "application/json"))  # type: str
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.AssetDto]

        if body is not None:
            _json = self._serialize.body(body, "object")
        else:
            _json = None

        request = build_assets_update_asset_request(
            id=id,
            content_type=content_type,
            json=_json,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("AssetDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_asset_structure_by_id(self, id: int, **kwargs: Any) -> _models.AssetStructureDto:
        """get_asset_structure_by_id.

        :param id: Required.
        :type id: int
        :return: AssetStructureDto
        :rtype: ~ignos.api.client.models.AssetStructureDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.AssetStructureDto]

        request = build_assets_get_asset_structure_by_id_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("AssetStructureDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_machines(self, **kwargs: Any) -> List[_models.MachineDto]:
        """list_machines.

        :return: list of MachineDto
        :rtype: list[~ignos.api.client.models.MachineDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MachineDto]]

        request = build_assets_list_machines_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MachineDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class AzureRegionsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`azure_regions` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_azure_regions(self, **kwargs: Any) -> List[_models.AzureRegionDto]:
        """list_azure_regions.

        :return: list of AzureRegionDto
        :rtype: list[~ignos.api.client.models.AzureRegionDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.AzureRegionDto]]

        request = build_azure_regions_list_azure_regions_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[AzureRegionDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class CdfOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`cdf` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get_cdf_config(self, **kwargs: Any) -> _models.CdfConfigDto:
        """Get CDF config.

        Get CDF config.

        :return: CdfConfigDto
        :rtype: ~ignos.api.client.models.CdfConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CdfConfigDto]

        request = build_cdf_get_cdf_config_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CdfConfigDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_cdf_config(
        self, body: Optional[_models.UpdateCdfConfig] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CdfConfigDto:
        """Update CDF config.

        Update CDF config.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateCdfConfig
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: CdfConfigDto
        :rtype: ~ignos.api.client.models.CdfConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_cdf_config(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CdfConfigDto:
        """Update CDF config.

        Update CDF config.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: CdfConfigDto
        :rtype: ~ignos.api.client.models.CdfConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_cdf_config(
        self, body: Optional[Union[_models.UpdateCdfConfig, IO]] = None, **kwargs: Any
    ) -> _models.CdfConfigDto:
        """Update CDF config.

        Update CDF config.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateCdfConfig or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: CdfConfigDto
        :rtype: ~ignos.api.client.models.CdfConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CdfConfigDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateCdfConfig")
            else:
                _json = None

        request = build_cdf_update_cdf_config_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CdfConfigDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class CountriesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`countries` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_countries(self, **kwargs: Any) -> List[_models.CountryDto]:
        """list_countries.

        :return: list of CountryDto
        :rtype: list[~ignos.api.client.models.CountryDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.CountryDto]]

        request = build_countries_list_countries_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[CountryDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class CustomerOrdersOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`customer_orders` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def upsert_customer_order(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.UpsertCustomerOrderRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Upsert a customer order.

        Upsert a customer order.

        :param id: Customer order id. Required.
        :type id: str
        :param body: Order details. Default value is None.
        :type body: ~ignos.api.client.models.UpsertCustomerOrderRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_customer_order(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Upsert a customer order.

        Upsert a customer order.

        :param id: Customer order id. Required.
        :type id: str
        :param body: Order details. Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_customer_order(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.UpsertCustomerOrderRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """Upsert a customer order.

        Upsert a customer order.

        :param id: Customer order id. Required.
        :type id: str
        :param body: Order details. Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpsertCustomerOrderRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpsertCustomerOrderRequest")
            else:
                _json = None

        request = build_customer_orders_upsert_customer_order_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def upsert_customer_order_line(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        line: int,
        body: Optional[_models.CustomerOrderLineDto] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Upsert a customer order line. Customer order must exist.

        Upsert a customer order line. Customer order must exist.

        :param id: Customer order id. Required.
        :type id: str
        :param line: Customer order line. Required.
        :type line: int
        :param body: Order line details. Default value is None.
        :type body: ~ignos.api.client.models.CustomerOrderLineDto
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_customer_order_line(  # pylint: disable=inconsistent-return-statements
        self, id: str, line: int, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Upsert a customer order line. Customer order must exist.

        Upsert a customer order line. Customer order must exist.

        :param id: Customer order id. Required.
        :type id: str
        :param line: Customer order line. Required.
        :type line: int
        :param body: Order line details. Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_customer_order_line(  # pylint: disable=inconsistent-return-statements
        self, id: str, line: int, body: Optional[Union[_models.CustomerOrderLineDto, IO]] = None, **kwargs: Any
    ) -> None:
        """Upsert a customer order line. Customer order must exist.

        Upsert a customer order line. Customer order must exist.

        :param id: Customer order id. Required.
        :type id: str
        :param line: Customer order line. Required.
        :type line: int
        :param body: Order line details. Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CustomerOrderLineDto or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CustomerOrderLineDto")
            else:
                _json = None

        request = build_customer_orders_upsert_customer_order_line_request(
            id=id,
            line=line,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})


class CustomersOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`customers` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def upsert_customer(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.UpsertCustomerRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Upsert a customer.

        Upsert a customer.

        :param id: Customer id. Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpsertCustomerRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_customer(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Upsert a customer.

        Upsert a customer.

        :param id: Customer id. Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_customer(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.UpsertCustomerRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """Upsert a customer.

        Upsert a customer.

        :param id: Customer id. Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpsertCustomerRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpsertCustomerRequest")
            else:
                _json = None

        request = build_customers_upsert_customer_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def delete_customer(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """Deletes a customer.

        Deletes a customer.

        :param id: Customer id. Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_customers_delete_customer_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_customers(
        self, *, page_size: int = 50, continuation_token_parameter: Optional[str] = None, **kwargs: Any
    ) -> _models.CustomerDtoPagedResult:
        """List customers.

        List customers.

        :keyword page_size: Page size. Default value is 50.
        :paramtype page_size: int
        :keyword continuation_token_parameter: Continuation token used for pagination. Default value is
         None.
        :paramtype continuation_token_parameter: str
        :return: CustomerDtoPagedResult
        :rtype: ~ignos.api.client.models.CustomerDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CustomerDtoPagedResult]

        request = build_customers_list_customers_request(
            page_size=page_size,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CustomerDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class WorkordersOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`workorders` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def upsert_workorder(  # pylint: disable=inconsistent-return-statements
        self, body: Optional[_models.UpsertWorkorder] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Create or update a workorder.

        Create or update a workorder.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpsertWorkorder
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_workorder(  # pylint: disable=inconsistent-return-statements
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Create or update a workorder.

        Create or update a workorder.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_workorder(  # pylint: disable=inconsistent-return-statements
        self, body: Optional[Union[_models.UpsertWorkorder, IO]] = None, **kwargs: Any
    ) -> None:
        """Create or update a workorder.

        Create or update a workorder.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpsertWorkorder or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpsertWorkorder")
            else:
                _json = None

        request = build_workorders_upsert_workorder_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_workorders(
        self,
        *,
        page_size: int = 50,
        search: Optional[str] = None,
        active_orders: Optional[bool] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.WorkorderListDtoPagedResult:
        """List work orders.

        List work orders.

        :keyword page_size: Number of items per page. Default 50. Use -1 for dynamic page size. Default
         value is 50.
        :paramtype page_size: int
        :keyword search: Searches work order ID using starts with. Default value is None.
        :paramtype search: str
        :keyword active_orders: Searches active or inactive work orders. Use null to ignore. Default
         value is None.
        :paramtype active_orders: bool
        :keyword continuation_token_parameter: Continuation token used for pagination. Default value is
         None.
        :paramtype continuation_token_parameter: str
        :return: WorkorderListDtoPagedResult
        :rtype: ~ignos.api.client.models.WorkorderListDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderListDtoPagedResult]

        request = build_workorders_list_workorders_request(
            page_size=page_size,
            search=search,
            active_orders=active_orders,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderListDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def upsert_workorder_traces(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.UpsertWorkorderTracesRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Create or update trace information for a workorder.

        Create or update trace information for a workorder.

        :param id: Workorder id. Required.
        :type id: str
        :param body: Trace details. Default value is None.
        :type body: ~ignos.api.client.models.UpsertWorkorderTracesRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_workorder_traces(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Create or update trace information for a workorder.

        Create or update trace information for a workorder.

        :param id: Workorder id. Required.
        :type id: str
        :param body: Trace details. Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_workorder_traces(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.UpsertWorkorderTracesRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """Create or update trace information for a workorder.

        Create or update trace information for a workorder.

        :param id: Workorder id. Required.
        :type id: str
        :param body: Trace details. Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpsertWorkorderTracesRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpsertWorkorderTracesRequest")
            else:
                _json = None

        request = build_workorders_upsert_workorder_traces_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def set_workorder_customer_order_reference(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.WorkorderCustomerOrderReferenceDto] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """set_workorder_customer_order_reference.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.WorkorderCustomerOrderReferenceDto
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_workorder_customer_order_reference(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """set_workorder_customer_order_reference.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_workorder_customer_order_reference(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.WorkorderCustomerOrderReferenceDto, IO]] = None, **kwargs: Any
    ) -> None:
        """set_workorder_customer_order_reference.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.WorkorderCustomerOrderReferenceDto or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "WorkorderCustomerOrderReferenceDto")
            else:
                _json = None

        request = build_workorders_set_workorder_customer_order_reference_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def check_resource_status(self, id: str, **kwargs: Any) -> _models.ResourceExistDto:
        """check_resource_status.

        :param id: Required.
        :type id: str
        :return: ResourceExistDto
        :rtype: ~ignos.api.client.models.ResourceExistDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ResourceExistDto]

        request = build_workorders_check_resource_status_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ResourceExistDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def start_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        operation: int,
        body: Optional[_models.StartWorkOperationRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.StartWorkOperationRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def start_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        operation: int,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def start_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        operation: int,
        body: Optional[Union[_models.StartWorkOperationRequest, IO]] = None,
        **kwargs: Any
    ) -> None:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.StartWorkOperationRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "StartWorkOperationRequest")
            else:
                _json = None

        request = build_workorders_start_workorder_operation_request(
            id=id,
            operation=operation,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def stop_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        operation: int,
        body: Optional[_models.StopWorkOperationRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.StopWorkOperationRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def stop_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        operation: int,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def stop_workorder_operation(  # pylint: disable=inconsistent-return-statements
        self, id: str, operation: int, body: Optional[Union[_models.StopWorkOperationRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.StopWorkOperationRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "StopWorkOperationRequest")
            else:
                _json = None

        request = build_workorders_stop_workorder_operation_request(
            id=id,
            operation=operation,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def register_workorder_operation_event(
        self,
        id: str,
        operation: int,
        body: Optional[_models.RegisterWorkorderOperationEventRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register a completed work order operation event.

        Register a completed work order operation event.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.RegisterWorkorderOperationEventRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def register_workorder_operation_event(
        self,
        id: str,
        operation: int,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register a completed work order operation event.

        Register a completed work order operation event.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def register_workorder_operation_event(
        self,
        id: str,
        operation: int,
        body: Optional[Union[_models.RegisterWorkorderOperationEventRequest, IO]] = None,
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register a completed work order operation event.

        Register a completed work order operation event.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.RegisterWorkorderOperationEventRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderOperationEventDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "RegisterWorkorderOperationEventRequest")
            else:
                _json = None

        request = build_workorders_register_workorder_operation_event_request(
            id=id,
            operation=operation,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderOperationEventDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def start_workorder_operation_v2(
        self,
        id: str,
        operation: int,
        body: Optional[_models.StartWorkOperationRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.StartWorkOperationRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def start_workorder_operation_v2(
        self,
        id: str,
        operation: int,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def start_workorder_operation_v2(
        self,
        id: str,
        operation: int,
        body: Optional[Union[_models.StartWorkOperationRequest, IO]] = None,
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register start work or setup on a workorder operation.

        Register start work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.StartWorkOperationRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderOperationEventDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "StartWorkOperationRequest")
            else:
                _json = None

        request = build_workorders_start_workorder_operation_v2_request(
            id=id,
            operation=operation,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderOperationEventDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def stop_workorder_operation_v2(
        self,
        id: str,
        operation: int,
        body: Optional[_models.StopWorkOperationRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.StopWorkOperationRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def stop_workorder_operation_v2(
        self,
        id: str,
        operation: int,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def stop_workorder_operation_v2(
        self, id: str, operation: int, body: Optional[Union[_models.StopWorkOperationRequest, IO]] = None, **kwargs: Any
    ) -> _models.WorkorderOperationEventDto:
        """Register end work or setup on a workorder operation.

        Register end work or setup on a workorder operation.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.StopWorkOperationRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderOperationEventDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "StopWorkOperationRequest")
            else:
                _json = None

        request = build_workorders_stop_workorder_operation_v2_request(
            id=id,
            operation=operation,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderOperationEventDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_active_workorder_operations(self, **kwargs: Any) -> List[_models.WorkorderOperationEventDto]:
        """list_active_workorder_operations.

        :return: list of WorkorderOperationEventDto
        :rtype: list[~ignos.api.client.models.WorkorderOperationEventDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.WorkorderOperationEventDto]]

        request = build_workorders_list_active_workorder_operations_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[WorkorderOperationEventDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_workorder_operation_event(  # pylint: disable=inconsistent-return-statements
        self, id: str, operation: int, event_id: str, **kwargs: Any
    ) -> None:
        """delete_workorder_operation_event.

        :param id: Required.
        :type id: str
        :param operation: Required.
        :type operation: int
        :param event_id: Required.
        :type event_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_workorders_delete_workorder_operation_event_request(
            id=id,
            operation=operation,
            event_id=event_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def get_workorder(self, id: str, **kwargs: Any) -> _models.WorkorderDto:
        """Get a specific work order.

        Get a specific work order.

        :param id: The work order ID. Required.
        :type id: str
        :return: WorkorderDto
        :rtype: ~ignos.api.client.models.WorkorderDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderDto]

        request = build_workorders_get_workorder_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_workorder(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """Deletes a specific work order.

        Deletes a specific work order.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_workorders_delete_workorder_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def get_workorder_operation_event(self, event_id: str, **kwargs: Any) -> _models.WorkorderOperationEventDto:
        """Get a specific work order operation event.

        Get a specific work order operation event.

        :param event_id: The ID of the event. Required.
        :type event_id: str
        :return: WorkorderOperationEventDto
        :rtype: ~ignos.api.client.models.WorkorderOperationEventDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.WorkorderOperationEventDto]

        request = build_workorders_get_workorder_operation_event_request(
            event_id=event_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("WorkorderOperationEventDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_workorder_operation_event_timestamps(  # pylint: disable=inconsistent-return-statements
        self,
        event_id: str,
        body: Optional[_models.UpdateWorkorderOperationEventTimestamps] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Update start time and/or end time of an existing work order operation event.

        Update start time and/or end time of an existing work order operation event.

        :param event_id: Required.
        :type event_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateWorkorderOperationEventTimestamps
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_workorder_operation_event_timestamps(  # pylint: disable=inconsistent-return-statements
        self, event_id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """Update start time and/or end time of an existing work order operation event.

        Update start time and/or end time of an existing work order operation event.

        :param event_id: Required.
        :type event_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_workorder_operation_event_timestamps(  # pylint: disable=inconsistent-return-statements
        self,
        event_id: str,
        body: Optional[Union[_models.UpdateWorkorderOperationEventTimestamps, IO]] = None,
        **kwargs: Any
    ) -> None:
        """Update start time and/or end time of an existing work order operation event.

        Update start time and/or end time of an existing work order operation event.

        :param event_id: Required.
        :type event_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateWorkorderOperationEventTimestamps or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateWorkorderOperationEventTimestamps")
            else:
                _json = None

        request = build_workorders_update_workorder_operation_event_timestamps_request(
            event_id=event_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def filter_workorder_operation_events(
        self,
        body: Optional[_models.FilterWorkorderOperationEvents] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> List[_models.WorkorderOperationEventDto]:
        """Filter work order operation events.

        Filter work order operation events.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.FilterWorkorderOperationEvents
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: list of WorkorderOperationEventDto
        :rtype: list[~ignos.api.client.models.WorkorderOperationEventDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def filter_workorder_operation_events(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> List[_models.WorkorderOperationEventDto]:
        """Filter work order operation events.

        Filter work order operation events.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: list of WorkorderOperationEventDto
        :rtype: list[~ignos.api.client.models.WorkorderOperationEventDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def filter_workorder_operation_events(
        self, body: Optional[Union[_models.FilterWorkorderOperationEvents, IO]] = None, **kwargs: Any
    ) -> List[_models.WorkorderOperationEventDto]:
        """Filter work order operation events.

        Filter work order operation events.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.FilterWorkorderOperationEvents or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: list of WorkorderOperationEventDto
        :rtype: list[~ignos.api.client.models.WorkorderOperationEventDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.WorkorderOperationEventDto]]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "FilterWorkorderOperationEvents")
            else:
                _json = None

        request = build_workorders_filter_workorder_operation_events_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[WorkorderOperationEventDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class ExternalOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`external` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_invites(self, **kwargs: Any) -> List[_models.InviteDto]:
        """list_invites.

        :return: list of InviteDto
        :rtype: list[~ignos.api.client.models.InviteDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.InviteDto]]

        request = build_external_list_invites_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[InviteDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def accept_supplier_invite(
        self,
        body: Optional[_models.AcceptSupplierInviteRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CompanyDto:
        """accept_supplier_invite.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.AcceptSupplierInviteRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: CompanyDto
        :rtype: ~ignos.api.client.models.CompanyDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def accept_supplier_invite(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CompanyDto:
        """accept_supplier_invite.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: CompanyDto
        :rtype: ~ignos.api.client.models.CompanyDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def accept_supplier_invite(
        self, body: Optional[Union[_models.AcceptSupplierInviteRequest, IO]] = None, **kwargs: Any
    ) -> _models.CompanyDto:
        """accept_supplier_invite.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.AcceptSupplierInviteRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: CompanyDto
        :rtype: ~ignos.api.client.models.CompanyDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CompanyDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "AcceptSupplierInviteRequest")
            else:
                _json = None

        request = build_external_accept_supplier_invite_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CompanyDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_companies(self, **kwargs: Any) -> List[_models.CompanyDto]:
        """list_companies.

        :return: list of CompanyDto
        :rtype: list[~ignos.api.client.models.CompanyDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.CompanyDto]]

        request = build_external_list_companies_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[CompanyDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class ExternalAccessOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`external_access` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_users(self, **kwargs: Any) -> List[_models.CompanyUserDto]:
        """list_users.

        :return: list of CompanyUserDto
        :rtype: list[~ignos.api.client.models.CompanyUserDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.CompanyUserDto]]

        request = build_external_access_list_users_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[CompanyUserDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_company_user(
        self,
        body: Optional[_models.CreateCompanyUserRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CompanyUserDto:
        """create_company_user.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateCompanyUserRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_company_user(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CompanyUserDto:
        """create_company_user.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_company_user(
        self, body: Optional[Union[_models.CreateCompanyUserRequest, IO]] = None, **kwargs: Any
    ) -> _models.CompanyUserDto:
        """create_company_user.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateCompanyUserRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CompanyUserDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateCompanyUserRequest")
            else:
                _json = None

        request = build_external_access_create_company_user_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CompanyUserDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_company_user(
        self,
        id: str,
        body: Optional[_models.UpdateCompanyUserRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CompanyUserDto:
        """update_company_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateCompanyUserRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_company_user(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CompanyUserDto:
        """update_company_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_company_user(
        self, id: str, body: Optional[Union[_models.UpdateCompanyUserRequest, IO]] = None, **kwargs: Any
    ) -> _models.CompanyUserDto:
        """update_company_user.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateCompanyUserRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: CompanyUserDto
        :rtype: ~ignos.api.client.models.CompanyUserDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.CompanyUserDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateCompanyUserRequest")
            else:
                _json = None

        request = build_external_access_update_company_user_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("CompanyUserDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_company_user(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_company_user.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_external_access_delete_company_user_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_roles(self, **kwargs: Any) -> List[_models.ExternalRoleDto]:
        """list_roles.

        :return: list of ExternalRoleDto
        :rtype: list[~ignos.api.client.models.ExternalRoleDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.ExternalRoleDto]]

        request = build_external_access_list_roles_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[ExternalRoleDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_company_customers(self, **kwargs: Any) -> List[_models.CompanyCustomerDto]:
        """list_company_customers.

        :return: list of CompanyCustomerDto
        :rtype: list[~ignos.api.client.models.CompanyCustomerDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.CompanyCustomerDto]]

        request = build_external_access_list_company_customers_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[CompanyCustomerDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_company_customer(  # pylint: disable=inconsistent-return-statements
        self, tenant_id: str, **kwargs: Any
    ) -> None:
        """delete_company_customer.

        :param tenant_id: Required.
        :type tenant_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_external_access_delete_company_customer_request(
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})


class ElectricalOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`electrical` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_electrical_source_types(self, **kwargs: Any) -> List[_models.IotTypeSourceDto]:
        """list_electrical_source_types.

        :return: list of IotTypeSourceDto
        :rtype: list[~ignos.api.client.models.IotTypeSourceDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.IotTypeSourceDto]]

        request = build_electrical_list_electrical_source_types_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[IotTypeSourceDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_electrical_data_configs(self, **kwargs: Any) -> List[_models.ElectricalIotConfigDto]:
        """list_electrical_data_configs.

        :return: list of ElectricalIotConfigDto
        :rtype: list[~ignos.api.client.models.ElectricalIotConfigDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.ElectricalIotConfigDto]]

        request = build_electrical_list_electrical_data_configs_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[ElectricalIotConfigDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_electrical_iot_config(
        self,
        body: Optional[_models.CreateElectricalIotConfig] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ElectricalIotConfigDto:
        """create_electrical_iot_config.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateElectricalIotConfig
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ElectricalIotConfigDto
        :rtype: ~ignos.api.client.models.ElectricalIotConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_electrical_iot_config(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ElectricalIotConfigDto:
        """create_electrical_iot_config.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: ElectricalIotConfigDto
        :rtype: ~ignos.api.client.models.ElectricalIotConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_electrical_iot_config(
        self, body: Optional[Union[_models.CreateElectricalIotConfig, IO]] = None, **kwargs: Any
    ) -> _models.ElectricalIotConfigDto:
        """create_electrical_iot_config.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateElectricalIotConfig or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: ElectricalIotConfigDto
        :rtype: ~ignos.api.client.models.ElectricalIotConfigDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ElectricalIotConfigDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateElectricalIotConfig")
            else:
                _json = None

        request = build_electrical_create_electrical_iot_config_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ElectricalIotConfigDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_electrical_iot_config(  # pylint: disable=inconsistent-return-statements
        self, type_id: str, id: str, **kwargs: Any
    ) -> None:
        """delete_electrical_iot_config.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_electrical_delete_electrical_iot_config_request(
            type_id=type_id,
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})


class WeldingOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`welding` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_welding_source_types(self, **kwargs: Any) -> List[_models.IotTypeSourceDto]:
        """list_welding_source_types.

        :return: list of IotTypeSourceDto
        :rtype: list[~ignos.api.client.models.IotTypeSourceDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.IotTypeSourceDto]]

        request = build_welding_list_welding_source_types_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[IotTypeSourceDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_electrical_data_configs(self, **kwargs: Any) -> List[JSON]:
        """list_electrical_data_configs.

        :return: list of JSON
        :rtype: list[JSON]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[JSON]]

        request = build_welding_list_electrical_data_configs_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[object]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def create_welding_iot_config(self, body: Optional[JSON] = None, **kwargs: Any) -> JSON:
        """create_welding_iot_config.

        :param body: Default value is None.
        :type body: JSON
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", "application/json"))  # type: str
        cls = kwargs.pop("cls", None)  # type: ClsType[JSON]

        if body is not None:
            _json = self._serialize.body(body, "object")
        else:
            _json = None

        request = build_welding_create_welding_iot_config_request(
            content_type=content_type,
            json=_json,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("object", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_welding_iot_config(  # pylint: disable=inconsistent-return-statements
        self, type_id: str, id: str, **kwargs: Any
    ) -> None:
        """delete_welding_iot_config.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_welding_delete_welding_iot_config_request(
            type_id=type_id,
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})


class LinksOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`links` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get_all_links(self, **kwargs: Any) -> List[_models.LinkDto]:
        """get_all_links.

        :return: list of LinkDto
        :rtype: list[~ignos.api.client.models.LinkDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.LinkDto]]

        request = build_links_get_all_links_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[LinkDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_global_link(
        self, body: Optional[_models.CreateLinkRequest] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.LinkDto:
        """create_global_link.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateLinkRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: LinkDto
        :rtype: ~ignos.api.client.models.LinkDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_global_link(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.LinkDto:
        """create_global_link.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: LinkDto
        :rtype: ~ignos.api.client.models.LinkDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_global_link(
        self, body: Optional[Union[_models.CreateLinkRequest, IO]] = None, **kwargs: Any
    ) -> _models.LinkDto:
        """create_global_link.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateLinkRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: LinkDto
        :rtype: ~ignos.api.client.models.LinkDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.LinkDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateLinkRequest")
            else:
                _json = None

        request = build_links_create_global_link_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("LinkDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class MachineAlarmsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`machine_alarms` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_machine_alarms(
        self,
        *,
        asset_id: Optional[int] = None,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmDto]:
        """list_machine_alarms.

        :keyword asset_id: Default value is None.
        :paramtype asset_id: int
        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :keyword end_time: Default value is None.
        :paramtype end_time: ~datetime.datetime
        :return: list of MachineAlarmDto
        :rtype: list[~ignos.api.client.models.MachineAlarmDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MachineAlarmDto]]

        request = build_machine_alarms_list_machine_alarms_request(
            asset_id=asset_id,
            start_time=start_time,
            end_time=end_time,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MachineAlarmDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_alarms_per_machine(
        self,
        *,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        asset_id: Optional[int] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmSummaryDto]:
        """list_alarms_per_machine.

        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :keyword end_time: Default value is None.
        :paramtype end_time: ~datetime.datetime
        :keyword asset_id: Default value is None.
        :paramtype asset_id: int
        :return: list of MachineAlarmSummaryDto
        :rtype: list[~ignos.api.client.models.MachineAlarmSummaryDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MachineAlarmSummaryDto]]

        request = build_machine_alarms_list_alarms_per_machine_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MachineAlarmSummaryDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_alarm_severity_occurrences(
        self,
        *,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        asset_id: Optional[int] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmSeverityOccurenceDto]:
        """list_alarm_severity_occurrences.

        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :keyword end_time: Default value is None.
        :paramtype end_time: ~datetime.datetime
        :keyword asset_id: Default value is None.
        :paramtype asset_id: int
        :return: list of MachineAlarmSeverityOccurenceDto
        :rtype: list[~ignos.api.client.models.MachineAlarmSeverityOccurenceDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MachineAlarmSeverityOccurenceDto]]

        request = build_machine_alarms_list_alarm_severity_occurrences_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MachineAlarmSeverityOccurenceDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_machine_alarm_details(self, id: int, **kwargs: Any) -> _models.MachineAlarmDetailsDto:
        """get_machine_alarm_details.

        :param id: Required.
        :type id: int
        :return: MachineAlarmDetailsDto
        :rtype: ~ignos.api.client.models.MachineAlarmDetailsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MachineAlarmDetailsDto]

        request = build_machine_alarms_get_machine_alarm_details_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MachineAlarmDetailsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class MachineUtilizationOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`machine_utilization` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get_machine_utilizations(
        self,
        *,
        asset_id: Optional[int] = None,
        favorites: bool = False,
        start_time_today: Optional[datetime.datetime] = None,
        **kwargs: Any
    ) -> _models.UtilizationDto:
        """get_machine_utilizations.

        :keyword asset_id: Default value is None.
        :paramtype asset_id: int
        :keyword favorites: Default value is False.
        :paramtype favorites: bool
        :keyword start_time_today: Default value is None.
        :paramtype start_time_today: ~datetime.datetime
        :return: UtilizationDto
        :rtype: ~ignos.api.client.models.UtilizationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.UtilizationDto]

        request = build_machine_utilization_get_machine_utilizations_request(
            asset_id=asset_id,
            favorites=favorites,
            start_time_today=start_time_today,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("UtilizationDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_utilization_data_point_for_machine(
        self, id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
    ) -> List[_models.NumericDataPointDto]:
        """get_utilization_data_point_for_machine.

        :param id: Required.
        :type id: int
        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :return: list of NumericDataPointDto
        :rtype: list[~ignos.api.client.models.NumericDataPointDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.NumericDataPointDto]]

        request = build_machine_utilization_get_utilization_data_point_for_machine_request(
            id=id,
            start_time=start_time,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[NumericDataPointDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_utilization_details_for_machine(
        self, id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
    ) -> _models.UtilizationDetailsDto:
        """get_utilization_details_for_machine.

        :param id: Required.
        :type id: int
        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :return: UtilizationDetailsDto
        :rtype: ~ignos.api.client.models.UtilizationDetailsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.UtilizationDetailsDto]

        request = build_machine_utilization_get_utilization_details_for_machine_request(
            id=id,
            start_time=start_time,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("UtilizationDetailsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_n_days_power_on_utilization_datapoints(
        self,
        *,
        id: Optional[int] = None,
        external_id: Optional[str] = None,
        n_days: Optional[int] = None,
        **kwargs: Any
    ) -> _models.PowerOnUtilizationList:
        """list_n_days_power_on_utilization_datapoints.

        :keyword id: Default value is None.
        :paramtype id: int
        :keyword external_id: Default value is None.
        :paramtype external_id: str
        :keyword n_days: Default value is None.
        :paramtype n_days: int
        :return: PowerOnUtilizationList
        :rtype: ~ignos.api.client.models.PowerOnUtilizationList
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.PowerOnUtilizationList]

        request = build_machine_utilization_list_n_days_power_on_utilization_datapoints_request(
            id=id,
            external_id=external_id,
            n_days=n_days,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("PowerOnUtilizationList", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_factory_utilization(self, **kwargs: Any) -> _models.PowerOnUtilizationDto:
        """get_factory_utilization.

        :return: PowerOnUtilizationDto
        :rtype: ~ignos.api.client.models.PowerOnUtilizationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.PowerOnUtilizationDto]

        request = build_machine_utilization_get_factory_utilization_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("PowerOnUtilizationDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_machine_states(
        self, id: int, *, start_time: Optional[datetime.datetime] = None, **kwargs: Any
    ) -> List[_models.MachineStateDatapoint]:
        """list_machine_states.

        :param id: Required.
        :type id: int
        :keyword start_time: Default value is None.
        :paramtype start_time: ~datetime.datetime
        :return: list of MachineStateDatapoint
        :rtype: list[~ignos.api.client.models.MachineStateDatapoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MachineStateDatapoint]]

        request = build_machine_utilization_list_machine_states_request(
            id=id,
            start_time=start_time,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MachineStateDatapoint]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class MeOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`me` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get_my_apps(self, **kwargs: Any) -> List[_models.UserAppDto]:
        """get_my_apps.

        :return: list of UserAppDto
        :rtype: list[~ignos.api.client.models.UserAppDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.UserAppDto]]

        request = build_me_get_my_apps_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[UserAppDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_current_user(self, **kwargs: Any) -> _models.UserDetailsDto:
        """get_current_user.

        :return: UserDetailsDto
        :rtype: ~ignos.api.client.models.UserDetailsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.UserDetailsDto]

        request = build_me_get_current_user_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("UserDetailsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class MeasurementFormSchemasOperations:  # pylint: disable=too-many-public-methods
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`measurement_form_schemas` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_measurment_form_schemas(
        self,
        *,
        page_size: int = 50,
        customer_id: Optional[str] = None,
        customer_name: Optional[str] = None,
        part_number: Optional[str] = None,
        part_revision: Optional[str] = None,
        drawing: Optional[str] = None,
        drawing_revision: Optional[str] = None,
        filter: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormListDtoPagedResult:
        """list_measurment_form_schemas.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword customer_id: Default value is None.
        :paramtype customer_id: str
        :keyword customer_name: Default value is None.
        :paramtype customer_name: str
        :keyword part_number: Default value is None.
        :paramtype part_number: str
        :keyword part_revision: Default value is None.
        :paramtype part_revision: str
        :keyword drawing: Default value is None.
        :paramtype drawing: str
        :keyword drawing_revision: Default value is None.
        :paramtype drawing_revision: str
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: MeasurementFormListDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormListDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormListDtoPagedResult]

        request = build_measurement_form_schemas_list_measurment_form_schemas_request(
            page_size=page_size,
            customer_id=customer_id,
            customer_name=customer_name,
            part_number=part_number,
            part_revision=part_revision,
            drawing=drawing,
            drawing_revision=drawing_revision,
            filter=filter,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormListDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measurement_form(
        self,
        body: Optional[_models.CreateMeasurementFormSchema] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchema
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measurement_form(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measurement_form(
        self, body: Optional[Union[_models.CreateMeasurementFormSchema, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchema or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasurementFormSchema")
            else:
                _json = None

        request = build_measurement_form_schemas_create_measurement_form_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_schema(self, id: str, **kwargs: Any) -> _models.MeasurementFormSchemaDto:
        """get_measurement_form_schema.

        :param id: Required.
        :type id: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaDto]

        request = build_measurement_form_schemas_get_measurement_form_schema_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measurement_form_schema(
        self,
        id: str,
        body: Optional[_models.UpdateMeasurementFormSchemaRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_measurement_form_schema.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormSchemaRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measurement_form_schema(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_measurement_form_schema.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measurement_form_schema(
        self, id: str, body: Optional[Union[_models.UpdateMeasurementFormSchemaRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_measurement_form_schema.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormSchemaRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasurementFormSchemaRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_update_measurement_form_schema_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_schema_grouped_elements(
        self,
        id: str,
        body: Optional[_models.UpdateSchemaGroupedElementsRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_schema_grouped_elements.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateSchemaGroupedElementsRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_schema_grouped_elements(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_schema_grouped_elements.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_schema_grouped_elements(
        self, id: str, body: Optional[Union[_models.UpdateSchemaGroupedElementsRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """update_schema_grouped_elements.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateSchemaGroupedElementsRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateSchemaGroupedElementsRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_update_schema_grouped_elements_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def upload_schema_drawing(
        self,
        id: str,
        body: Optional[_models.UploadDrawingRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_drawing.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UploadDrawingRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upload_schema_drawing(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_drawing.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upload_schema_drawing(
        self, id: str, body: Optional[Union[_models.UploadDrawingRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_drawing.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UploadDrawingRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UploadDrawingRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_upload_schema_drawing_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def upload_schema_attachment(
        self,
        id: str,
        body: Optional[_models.UploadRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_attachment.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UploadRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upload_schema_attachment(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_attachment.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upload_schema_attachment(
        self, id: str, body: Optional[Union[_models.UploadRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaDto:
        """upload_schema_attachment.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UploadRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UploadRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_upload_schema_attachment_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_import_status(self, id: str, **kwargs: Any) -> _models.MeasurementFormImportStatusDto:
        """get_measurement_form_import_status.

        :param id: Required.
        :type id: str
        :return: MeasurementFormImportStatusDto
        :rtype: ~ignos.api.client.models.MeasurementFormImportStatusDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormImportStatusDto]

        request = build_measurement_form_schemas_get_measurement_form_import_status_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormImportStatusDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_schema_element(
        self,
        schema_id: str,
        body: Optional[_models.CreateSchemaElement] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormElementDto:
        """create_schema_element.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateSchemaElement
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormElementDto
        :rtype: ~ignos.api.client.models.MeasurementFormElementDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_schema_element(
        self, schema_id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormElementDto:
        """create_schema_element.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormElementDto
        :rtype: ~ignos.api.client.models.MeasurementFormElementDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_schema_element(
        self, schema_id: str, body: Optional[Union[_models.CreateSchemaElement, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormElementDto:
        """create_schema_element.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateSchemaElement or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormElementDto
        :rtype: ~ignos.api.client.models.MeasurementFormElementDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormElementDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateSchemaElement")
            else:
                _json = None

        request = build_measurement_form_schemas_create_schema_element_request(
            schema_id=schema_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormElementDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_linkable_measurement_form_schemas(
        self,
        schema_id: str,
        *,
        page_size: int = 50,
        filter: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormListDtoPagedResult:
        """list_linkable_measurement_form_schemas.

        :param schema_id: Required.
        :type schema_id: str
        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: MeasurementFormListDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormListDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormListDtoPagedResult]

        request = build_measurement_form_schemas_list_linkable_measurement_form_schemas_request(
            schema_id=schema_id,
            page_size=page_size,
            filter=filter,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormListDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measurement_form_schema_link(
        self,
        schema_id: str,
        body: Optional[_models.CreateMeasurementFormSchemaLinkRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form_schema_link.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchemaLinkRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measurement_form_schema_link(
        self, schema_id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form_schema_link.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measurement_form_schema_link(
        self,
        schema_id: str,
        body: Optional[Union[_models.CreateMeasurementFormSchemaLinkRequest, IO]] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """create_measurement_form_schema_link.

        :param schema_id: Required.
        :type schema_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchemaLinkRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasurementFormSchemaLinkRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_create_measurement_form_schema_link_request(
            schema_id=schema_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measurement_form_schema_link(  # pylint: disable=inconsistent-return-statements
        self, schema_id: str, linked_schema_id: str, **kwargs: Any
    ) -> None:
        """delete_measurement_form_schema_link.

        :param schema_id: Required.
        :type schema_id: str
        :param linked_schema_id: Required.
        :type linked_schema_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_measurement_form_schema_link_request(
            schema_id=schema_id,
            linked_schema_id=linked_schema_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def release_schema(self, schema_id: str, **kwargs: Any) -> _models.MeasurementFormDto:
        """release_schema.

        :param schema_id: Required.
        :type schema_id: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        request = build_measurement_form_schemas_release_schema_request(
            schema_id=schema_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def revoke_schema(self, schema_id: str, **kwargs: Any) -> _models.MeasurementFormDto:
        """revoke_schema.

        :param schema_id: Required.
        :type schema_id: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        request = build_measurement_form_schemas_revoke_schema_request(
            schema_id=schema_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def create_draft_version(self, schema_id: str, **kwargs: Any) -> _models.MeasurementFormDto:
        """create_draft_version.

        :param schema_id: Required.
        :type schema_id: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        request = build_measurement_form_schemas_create_draft_version_request(
            schema_id=schema_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def is_mapping_from_previous_version_required(self, schema_id: str, **kwargs: Any) -> bool:
        """is_mapping_from_previous_version_required.

        :param schema_id: Required.
        :type schema_id: str
        :return: bool
        :rtype: bool
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[bool]

        request = build_measurement_form_schemas_is_mapping_from_previous_version_required_request(
            schema_id=schema_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("bool", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_settings(self, **kwargs: Any) -> _models.MeasurementFormSettingsDto:
        """get_measurement_form_settings.

        :return: MeasurementFormSettingsDto
        :rtype: ~ignos.api.client.models.MeasurementFormSettingsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSettingsDto]

        request = build_measurement_form_schemas_get_measurement_form_settings_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSettingsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measurement_form_settings(
        self,
        body: Optional[_models.UpdateMeasurementFormSettings] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSettingsDto:
        """update_measurement_form_settings.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormSettings
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSettingsDto
        :rtype: ~ignos.api.client.models.MeasurementFormSettingsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measurement_form_settings(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSettingsDto:
        """update_measurement_form_settings.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSettingsDto
        :rtype: ~ignos.api.client.models.MeasurementFormSettingsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measurement_form_settings(
        self, body: Optional[Union[_models.UpdateMeasurementFormSettings, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSettingsDto:
        """update_measurement_form_settings.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormSettings or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSettingsDto
        :rtype: ~ignos.api.client.models.MeasurementFormSettingsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSettingsDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasurementFormSettings")
            else:
                _json = None

        request = build_measurement_form_schemas_update_measurement_form_settings_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSettingsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_measurement_form_mappings_from_schema(
        self, *, target_id: str, **kwargs: Any
    ) -> List[_models.MeasurementFormMappingDto]:
        """list_measurement_form_mappings_from_schema.

        :keyword target_id: Required.
        :paramtype target_id: str
        :return: list of MeasurementFormMappingDto
        :rtype: list[~ignos.api.client.models.MeasurementFormMappingDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasurementFormMappingDto]]

        request = build_measurement_form_schemas_list_measurement_form_mappings_from_schema_request(
            target_id=target_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasurementFormMappingDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measurement_form_mapping(
        self,
        body: Optional[_models.CreateMeasurementFormMapping] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> Union[_models.MeasurementFormMappingDto, _models.ProblemDetails]:
        """create_measurement_form_mapping.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormMapping
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormMappingDto or ProblemDetails
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto or
         ~ignos.api.client.models.ProblemDetails
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measurement_form_mapping(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> Union[_models.MeasurementFormMappingDto, _models.ProblemDetails]:
        """create_measurement_form_mapping.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormMappingDto or ProblemDetails
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto or
         ~ignos.api.client.models.ProblemDetails
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measurement_form_mapping(
        self, body: Optional[Union[_models.CreateMeasurementFormMapping, IO]] = None, **kwargs: Any
    ) -> Union[_models.MeasurementFormMappingDto, _models.ProblemDetails]:
        """create_measurement_form_mapping.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormMapping or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormMappingDto or ProblemDetails
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto or
         ~ignos.api.client.models.ProblemDetails
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[Union[_models.MeasurementFormMappingDto, _models.ProblemDetails]]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasurementFormMapping")
            else:
                _json = None

        request = build_measurement_form_schemas_create_measurement_form_mapping_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [201, 400, 409]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if response.status_code == 201:
            deserialized = self._deserialize("MeasurementFormMappingDto", pipeline_response)

        if response.status_code == 400:
            deserialized = self._deserialize("ProblemDetails", pipeline_response)

        if response.status_code == 409:
            deserialized = self._deserialize("ProblemDetails", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_mapping(self, id: str, **kwargs: Any) -> _models.MeasurementFormMappingDto:
        """get_measurement_form_mapping.

        :param id: Required.
        :type id: str
        :return: MeasurementFormMappingDto
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormMappingDto]

        request = build_measurement_form_schemas_get_measurement_form_mapping_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormMappingDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measurement_form_mapping(  # pylint: disable=inconsistent-return-statements
        self, id: str, **kwargs: Any
    ) -> None:
        """delete_measurement_form_mapping.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_measurement_form_mapping_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def set_measurement_form_mapping_balloons(
        self,
        id: str,
        body: Optional[List[_models.MeasurementFormBalloonMappingRequestDto]] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormMappingDto:
        """set_measurement_form_mapping_balloons.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: list[~ignos.api.client.models.MeasurementFormBalloonMappingRequestDto]
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormMappingDto
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_measurement_form_mapping_balloons(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormMappingDto:
        """set_measurement_form_mapping_balloons.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormMappingDto
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_measurement_form_mapping_balloons(
        self,
        id: str,
        body: Optional[Union[List[_models.MeasurementFormBalloonMappingRequestDto], IO]] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormMappingDto:
        """set_measurement_form_mapping_balloons.

        :param id: Required.
        :type id: str
        :param body: Is either a list type or a IO type. Default value is None.
        :type body: list[~ignos.api.client.models.MeasurementFormBalloonMappingRequestDto] or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormMappingDto
        :rtype: ~ignos.api.client.models.MeasurementFormMappingDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormMappingDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "[MeasurementFormBalloonMappingRequestDto]")
            else:
                _json = None

        request = build_measurement_form_schemas_set_measurement_form_mapping_balloons_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormMappingDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_mapping_suggestion(
        self, *, target_id: str, source_id: Optional[str] = None, **kwargs: Any
    ) -> _models.MeasurementFormMappingSuggestionDto:
        """get_measurement_form_mapping_suggestion.

        :keyword target_id: Required.
        :paramtype target_id: str
        :keyword source_id: Default value is None.
        :paramtype source_id: str
        :return: MeasurementFormMappingSuggestionDto
        :rtype: ~ignos.api.client.models.MeasurementFormMappingSuggestionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormMappingSuggestionDto]

        request = build_measurement_form_schemas_get_measurement_form_mapping_suggestion_request(
            target_id=target_id,
            source_id=source_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormMappingSuggestionDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def import_measurement_form_schema(
        self,
        body: Optional[_models.ImportMeasurementFormSchema] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """Custom api for initial import. Not to be used more than once per customer.

        Custom api for initial import. Not to be used more than once per customer.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.ImportMeasurementFormSchema
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def import_measurement_form_schema(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """Custom api for initial import. Not to be used more than once per customer.

        Custom api for initial import. Not to be used more than once per customer.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def import_measurement_form_schema(
        self, body: Optional[Union[_models.ImportMeasurementFormSchema, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormDto:
        """Custom api for initial import. Not to be used more than once per customer.

        Custom api for initial import. Not to be used more than once per customer.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.ImportMeasurementFormSchema or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "ImportMeasurementFormSchema")
            else:
                _json = None

        request = build_measurement_form_schemas_import_measurement_form_schema_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_measurement_form_needs(
        self,
        *,
        page_size: int = 50,
        customer_id: Optional[str] = None,
        customer_name: Optional[str] = None,
        part_number: Optional[str] = None,
        part_revision: Optional[str] = None,
        drawing: Optional[str] = None,
        drawing_revision: Optional[str] = None,
        filter: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        only_without_drawing_url: Optional[bool] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormNeedDtoPagedResult:
        """list_measurement_form_needs.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword customer_id: Default value is None.
        :paramtype customer_id: str
        :keyword customer_name: Default value is None.
        :paramtype customer_name: str
        :keyword part_number: Default value is None.
        :paramtype part_number: str
        :keyword part_revision: Default value is None.
        :paramtype part_revision: str
        :keyword drawing: Default value is None.
        :paramtype drawing: str
        :keyword drawing_revision: Default value is None.
        :paramtype drawing_revision: str
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :keyword only_without_drawing_url: Default value is None.
        :paramtype only_without_drawing_url: bool
        :return: MeasurementFormNeedDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormNeedDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormNeedDtoPagedResult]

        request = build_measurement_form_schemas_list_measurement_form_needs_request(
            page_size=page_size,
            customer_id=customer_id,
            customer_name=customer_name,
            part_number=part_number,
            part_revision=part_revision,
            drawing=drawing,
            drawing_revision=drawing_revision,
            filter=filter,
            continuation_token_parameter=continuation_token_parameter,
            only_without_drawing_url=only_without_drawing_url,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormNeedDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def set_measurement_form_need_user(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.SetMeasurementFormNeedUserRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """set_measurement_form_need_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormNeedUserRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_measurement_form_need_user(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """set_measurement_form_need_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_measurement_form_need_user(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.SetMeasurementFormNeedUserRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """set_measurement_form_need_user.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormNeedUserRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SetMeasurementFormNeedUserRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_set_measurement_form_need_user_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def set_measurement_form_need_as_not_needed(
        self,
        id: str,
        body: Optional[_models.SetMeasurementFormNeedAsNotNeededRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurement_form_need_as_not_needed.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormNeedAsNotNeededRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_measurement_form_need_as_not_needed(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurement_form_need_as_not_needed.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_measurement_form_need_as_not_needed(
        self, id: str, body: Optional[Union[_models.SetMeasurementFormNeedAsNotNeededRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurement_form_need_as_not_needed.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormNeedAsNotNeededRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaNotNeededDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SetMeasurementFormNeedAsNotNeededRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_set_measurement_form_need_as_not_needed_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaNotNeededDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def create_measurement_form_schema_from_need(self, id: str, **kwargs: Any) -> _models.MeasurementFormDto:
        """create_measurement_form_schema_from_need.

        :param id: Required.
        :type id: str
        :return: MeasurementFormDto
        :rtype: ~ignos.api.client.models.MeasurementFormDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormDto]

        request = build_measurement_form_schemas_create_measurement_form_schema_from_need_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def upload_need_drawing(
        self,
        id: str,
        body: Optional[_models.UploadDrawingRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormNeedDto:
        """upload_need_drawing.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UploadDrawingRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormNeedDto
        :rtype: ~ignos.api.client.models.MeasurementFormNeedDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upload_need_drawing(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormNeedDto:
        """upload_need_drawing.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormNeedDto
        :rtype: ~ignos.api.client.models.MeasurementFormNeedDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upload_need_drawing(
        self, id: str, body: Optional[Union[_models.UploadDrawingRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormNeedDto:
        """upload_need_drawing.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UploadDrawingRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormNeedDto
        :rtype: ~ignos.api.client.models.MeasurementFormNeedDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormNeedDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UploadDrawingRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_upload_need_drawing_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormNeedDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_measurment_form_schemas_not_needed(
        self,
        *,
        page_size: int = 50,
        customer_id: Optional[str] = None,
        customer_name: Optional[str] = None,
        part_number: Optional[str] = None,
        part_revision: Optional[str] = None,
        drawing: Optional[str] = None,
        drawing_revision: Optional[str] = None,
        filter: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDtoPagedResult:
        """list_measurment_form_schemas_not_needed.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword customer_id: Default value is None.
        :paramtype customer_id: str
        :keyword customer_name: Default value is None.
        :paramtype customer_name: str
        :keyword part_number: Default value is None.
        :paramtype part_number: str
        :keyword part_revision: Default value is None.
        :paramtype part_revision: str
        :keyword drawing: Default value is None.
        :paramtype drawing: str
        :keyword drawing_revision: Default value is None.
        :paramtype drawing_revision: str
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: MeasurementFormSchemaNotNeededDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaNotNeededDtoPagedResult]

        request = build_measurement_form_schemas_list_measurment_form_schemas_not_needed_request(
            page_size=page_size,
            customer_id=customer_id,
            customer_name=customer_name,
            part_number=part_number,
            part_revision=part_revision,
            drawing=drawing,
            drawing_revision=drawing_revision,
            filter=filter,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaNotNeededDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def set_measurment_form_schemas_not_needed(
        self,
        body: Optional[_models.SetMeasurementFormSchemasNotNeeded] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurment_form_schemas_not_needed.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormSchemasNotNeeded
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_measurment_form_schemas_not_needed(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurment_form_schemas_not_needed.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_measurment_form_schemas_not_needed(
        self, body: Optional[Union[_models.SetMeasurementFormSchemasNotNeeded, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormSchemaNotNeededDto:
        """set_measurment_form_schemas_not_needed.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormSchemasNotNeeded or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormSchemaNotNeededDto
        :rtype: ~ignos.api.client.models.MeasurementFormSchemaNotNeededDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormSchemaNotNeededDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SetMeasurementFormSchemasNotNeeded")
            else:
                _json = None

        request = build_measurement_form_schemas_set_measurment_form_schemas_not_needed_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormSchemaNotNeededDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measurment_form_schemas_not_needed(  # pylint: disable=inconsistent-return-statements
        self, id: str, **kwargs: Any
    ) -> None:
        """delete_measurment_form_schemas_not_needed.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_measurment_form_schemas_not_needed_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def delete_imported_measurment_schemas(  # pylint: disable=inconsistent-return-statements
        self, **kwargs: Any
    ) -> None:
        """delete_imported_measurment_schemas.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_imported_measurment_schemas_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_resource_types_for_needs_generator(
        self, **kwargs: Any
    ) -> List[_models.MeasurementFormResourceTypeGeneratorDto]:
        """list_resource_types_for_needs_generator.

        :return: list of MeasurementFormResourceTypeGeneratorDto
        :rtype: list[~ignos.api.client.models.MeasurementFormResourceTypeGeneratorDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasurementFormResourceTypeGeneratorDto]]

        request = build_measurement_form_schemas_list_resource_types_for_needs_generator_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasurementFormResourceTypeGeneratorDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def add_resource_type_for_needs_generator(
        self,
        body: Optional[_models.AddResourceTypeForNeedsGenerator] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormResourceTypeGeneratorDto:
        """add_resource_type_for_needs_generator.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.AddResourceTypeForNeedsGenerator
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormResourceTypeGeneratorDto
        :rtype: ~ignos.api.client.models.MeasurementFormResourceTypeGeneratorDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def add_resource_type_for_needs_generator(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormResourceTypeGeneratorDto:
        """add_resource_type_for_needs_generator.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormResourceTypeGeneratorDto
        :rtype: ~ignos.api.client.models.MeasurementFormResourceTypeGeneratorDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def add_resource_type_for_needs_generator(
        self, body: Optional[Union[_models.AddResourceTypeForNeedsGenerator, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormResourceTypeGeneratorDto:
        """add_resource_type_for_needs_generator.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.AddResourceTypeForNeedsGenerator or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormResourceTypeGeneratorDto
        :rtype: ~ignos.api.client.models.MeasurementFormResourceTypeGeneratorDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormResourceTypeGeneratorDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "AddResourceTypeForNeedsGenerator")
            else:
                _json = None

        request = build_measurement_form_schemas_add_resource_type_for_needs_generator_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormResourceTypeGeneratorDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_resource_type_for_needs_generator(  # pylint: disable=inconsistent-return-statements
        self, resource_type: str, **kwargs: Any
    ) -> None:
        """delete_resource_type_for_needs_generator.

        :param resource_type: Required.
        :type resource_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_resource_type_for_needs_generator_request(
            resource_type=resource_type,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_frequencies(self, **kwargs: Any) -> List[_models.MeasurementFrequencyDto]:
        """list_frequencies.

        :return: list of MeasurementFrequencyDto
        :rtype: list[~ignos.api.client.models.MeasurementFrequencyDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasurementFrequencyDto]]

        request = build_measurement_form_schemas_list_frequencies_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasurementFrequencyDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_instance_schema_feedback(
        self,
        *,
        page_size: int = 50,
        search: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SchemaFeedbackDtoPagedResult:
        """get_measurement_form_instance_schema_feedback.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword search: Default value is None.
        :paramtype search: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: SchemaFeedbackDtoPagedResult
        :rtype: ~ignos.api.client.models.SchemaFeedbackDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SchemaFeedbackDtoPagedResult]

        request = build_measurement_form_schemas_get_measurement_form_instance_schema_feedback_request(
            page_size=page_size,
            search=search,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SchemaFeedbackDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def set_measurement_form_instance_schema_feedback_user(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.SetMeasurementFormSchemaFeedbackUserRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """set_measurement_form_instance_schema_feedback_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormSchemaFeedbackUserRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def set_measurement_form_instance_schema_feedback_user(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """set_measurement_form_instance_schema_feedback_user.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def set_measurement_form_instance_schema_feedback_user(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[Union[_models.SetMeasurementFormSchemaFeedbackUserRequest, IO]] = None,
        **kwargs: Any
    ) -> None:
        """set_measurement_form_instance_schema_feedback_user.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SetMeasurementFormSchemaFeedbackUserRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SetMeasurementFormSchemaFeedbackUserRequest")
            else:
                _json = None

        request = build_measurement_form_schemas_set_measurement_form_instance_schema_feedback_user_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def delete_measurement_form_instance_schema_feedback(  # pylint: disable=inconsistent-return-statements
        self, id: str, **kwargs: Any
    ) -> None:
        """delete_measurement_form_instance_schema_feedback.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_form_schemas_delete_measurement_form_instance_schema_feedback_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})


class MeasurementFormsInstancesOperations:  # pylint: disable=too-many-public-methods
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`measurement_forms_instances` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_active_measurement_forms(
        self,
        *,
        page_size: int = 50,
        search: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormInstanceOverviewDtoPagedResult:
        """list_active_measurement_forms.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword search: Default value is None.
        :paramtype search: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: MeasurementFormInstanceOverviewDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceOverviewDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceOverviewDtoPagedResult]

        request = build_measurement_forms_instances_list_active_measurement_forms_request(
            page_size=page_size,
            search=search,
            continuation_token_parameter=continuation_token_parameter,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceOverviewDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_measurement_forms_by_status(
        self,
        *,
        status: Optional[Union[str, "_models.MeasurementFormInstanceStatus"]] = None,
        page_size: int = 50,
        search: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasurementFormInstanceDtoPagedResult:
        """list_measurement_forms_by_status.

        :keyword status: Known values are: "Active", "Cancelled", "Completed", and "Approved". Default
         value is None.
        :paramtype status: str or ~ignos.api.client.models.MeasurementFormInstanceStatus
        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword search: Default value is None.
        :paramtype search: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: MeasurementFormInstanceDtoPagedResult
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDtoPagedResult]

        request = build_measurement_forms_instances_list_measurement_forms_by_status_request(
            status=status,
            page_size=page_size,
            search=search,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_instance(
        self, id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """get_measurement_form_instance.

        :param id: Required.
        :type id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_get_measurement_form_instance_request(
            id=id,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measurement_form_instance(
        self,
        id: str,
        body: Optional[_models.CreateMeasurementFormInstanceRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """create_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormInstanceRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measurement_form_instance(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """create_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measurement_form_instance(
        self, id: str, body: Optional[Union[_models.CreateMeasurementFormInstanceRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """create_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormInstanceRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasurementFormInstanceRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_create_measurement_form_instance_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measurement_form_instance(
        self,
        id: str,
        body: Optional[_models.UpdateMeasurementFormInstanceRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """update_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormInstanceRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measurement_form_instance(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """update_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measurement_form_instance(
        self, id: str, body: Optional[Union[_models.UpdateMeasurementFormInstanceRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """update_measurement_form_instance.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasurementFormInstanceRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasurementFormInstanceRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_update_measurement_form_instance_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def complete_measurement_form_instance(
        self, id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceDto:
        """complete_measurement_form_instance.

        :param id: Required.
        :type id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_complete_measurement_form_instance_request(
            id=id,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def reactivate_measurement_form_instance(self, id: str, **kwargs: Any) -> _models.MeasurementFormInstanceDto:
        """reactivate_measurement_form_instance.

        :param id: Required.
        :type id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_reactivate_measurement_form_instance_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def approve_measurement_form_instance(self, id: str, **kwargs: Any) -> _models.MeasurementFormInstanceDto:
        """approve_measurement_form_instance.

        :param id: Required.
        :type id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_approve_measurement_form_instance_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def disapprove_measurement_form_instance(self, id: str, **kwargs: Any) -> _models.MeasurementFormInstanceDto:
        """disapprove_measurement_form_instance.

        :param id: Required.
        :type id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_disapprove_measurement_form_instance_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def cancel_measurement_form_instance(self, id: str, **kwargs: Any) -> _models.MeasurementFormInstanceDto:
        """cancel_measurement_form_instance.

        :param id: Required.
        :type id: str
        :return: MeasurementFormInstanceDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceDto]

        request = build_measurement_forms_instances_cancel_measurement_form_instance_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measurement_form_instance_schema(
        self, id: str, schema_id: str, *, sequence: Optional[str] = None, tenant_id: Optional[str] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceSchemaDto:
        """get_measurement_form_instance_schema.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :keyword sequence: Default value is None.
        :paramtype sequence: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: MeasurementFormInstanceSchemaDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceSchemaDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceSchemaDto]

        request = build_measurement_forms_instances_get_measurement_form_instance_schema_request(
            id=id,
            schema_id=schema_id,
            sequence=sequence,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceSchemaDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_workorder_measurement_form_progress(
        self, id: str, *, tenant_id: Optional[str] = None, **kwargs: Any
    ) -> _models.MeasurementFormInstanceProgressDto:
        """get_workorder_measurement_form_progress.

        :param id: Required.
        :type id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: MeasurementFormInstanceProgressDto
        :rtype: ~ignos.api.client.models.MeasurementFormInstanceProgressDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasurementFormInstanceProgressDto]

        request = build_measurement_forms_instances_get_workorder_measurement_form_progress_request(
            id=id,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasurementFormInstanceProgressDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_audit_log(
        self,
        id: str,
        *,
        tenant_id: Optional[str] = None,
        schema_id: Optional[str] = None,
        sequence: Optional[str] = None,
        element_id: Optional[str] = None,
        **kwargs: Any
    ) -> List[_models.MeasurementFormElementValueAuditDto]:
        """get_audit_log.

        :param id: Required.
        :type id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword schema_id: Default value is None.
        :paramtype schema_id: str
        :keyword sequence: Default value is None.
        :paramtype sequence: str
        :keyword element_id: Default value is None.
        :paramtype element_id: str
        :return: list of MeasurementFormElementValueAuditDto
        :rtype: list[~ignos.api.client.models.MeasurementFormElementValueAuditDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasurementFormElementValueAuditDto]]

        request = build_measurement_forms_instances_get_audit_log_request(
            id=id,
            tenant_id=tenant_id,
            schema_id=schema_id,
            sequence=sequence,
            element_id=element_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasurementFormElementValueAuditDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_validation_rules(self, **kwargs: Any) -> List[_models.ValidationRuleDto]:
        """get_validation_rules.

        :return: list of ValidationRuleDto
        :rtype: list[~ignos.api.client.models.ValidationRuleDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.ValidationRuleDto]]

        request = build_measurement_forms_instances_get_validation_rules_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[ValidationRuleDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def set_cmm_element(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        *,
        schema_id: Optional[str] = None,
        element_id: Optional[str] = None,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """set_cmm_element.

        :param id: Required.
        :type id: str
        :keyword schema_id: Default value is None.
        :paramtype schema_id: str
        :keyword element_id: Default value is None.
        :paramtype element_id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_forms_instances_set_cmm_element_request(
            id=id,
            schema_id=schema_id,
            element_id=element_id,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def remove_cmm_element(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        *,
        schema_id: Optional[str] = None,
        element_id: Optional[str] = None,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """remove_cmm_element.

        :param id: Required.
        :type id: str
        :keyword schema_id: Default value is None.
        :paramtype schema_id: str
        :keyword element_id: Default value is None.
        :paramtype element_id: str
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_forms_instances_remove_cmm_element_request(
            id=id,
            schema_id=schema_id,
            element_id=element_id,
            tenant_id=tenant_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def save_value(
        self,
        id: str,
        body: Optional[_models.SaveValueRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_value.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SaveValueRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def save_value(
        self,
        id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_value.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def save_value(
        self,
        id: str,
        body: Optional[Union[_models.SaveValueRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_value.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SaveValueRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SaveValueResponseDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SaveValueRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_save_value_request(
            id=id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SaveValueResponseDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def save_tool(
        self,
        id: str,
        body: Optional[_models.SaveToolRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SaveToolRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def save_tool(
        self,
        id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def save_tool(
        self,
        id: str,
        body: Optional[Union[_models.SaveToolRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_tool.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SaveToolRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SaveValueResponseDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SaveToolRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_save_tool_request(
            id=id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SaveValueResponseDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def save_comment(
        self,
        id: str,
        body: Optional[_models.SaveCommentRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_comment.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SaveCommentRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def save_comment(
        self,
        id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_comment.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def save_comment(
        self,
        id: str,
        body: Optional[Union[_models.SaveCommentRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SaveValueResponseDto:
        """save_comment.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SaveCommentRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: SaveValueResponseDto
        :rtype: ~ignos.api.client.models.SaveValueResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SaveValueResponseDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SaveCommentRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_save_comment_request(
            id=id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SaveValueResponseDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def batch_insert_values(
        self,
        id: str,
        body: Optional[_models.BatchInsertValueRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.BatchInsertValuesResponseDto:
        """batch_insert_values.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.BatchInsertValueRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: BatchInsertValuesResponseDto
        :rtype: ~ignos.api.client.models.BatchInsertValuesResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def batch_insert_values(
        self,
        id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.BatchInsertValuesResponseDto:
        """batch_insert_values.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: BatchInsertValuesResponseDto
        :rtype: ~ignos.api.client.models.BatchInsertValuesResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def batch_insert_values(
        self,
        id: str,
        body: Optional[Union[_models.BatchInsertValueRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.BatchInsertValuesResponseDto:
        """batch_insert_values.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.BatchInsertValueRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: BatchInsertValuesResponseDto
        :rtype: ~ignos.api.client.models.BatchInsertValuesResponseDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.BatchInsertValuesResponseDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "BatchInsertValueRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_batch_insert_values_request(
            id=id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("BatchInsertValuesResponseDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def save_measurement_form_instance_schema_comment(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        schema_id: str,
        body: Optional[_models.SaveMeasurementFormInstanceSchemaCommentRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """save_measurement_form_instance_schema_comment.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.SaveMeasurementFormInstanceSchemaCommentRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def save_measurement_form_instance_schema_comment(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        schema_id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """save_measurement_form_instance_schema_comment.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def save_measurement_form_instance_schema_comment(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        schema_id: str,
        body: Optional[Union[_models.SaveMeasurementFormInstanceSchemaCommentRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """save_measurement_form_instance_schema_comment.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.SaveMeasurementFormInstanceSchemaCommentRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "SaveMeasurementFormInstanceSchemaCommentRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_save_measurement_form_instance_schema_comment_request(
            id=id,
            schema_id=schema_id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def create_measurement_form_instance_schema_feedback(
        self,
        id: str,
        schema_id: str,
        body: Optional[_models.CreateMeasurementFormSchemaFeedbackRequest] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SchemaFeedbackDto:
        """create_measurement_form_instance_schema_feedback.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchemaFeedbackRequest
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SchemaFeedbackDto
        :rtype: ~ignos.api.client.models.SchemaFeedbackDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measurement_form_instance_schema_feedback(
        self,
        id: str,
        schema_id: str,
        body: Optional[IO] = None,
        *,
        tenant_id: Optional[str] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SchemaFeedbackDto:
        """create_measurement_form_instance_schema_feedback.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: SchemaFeedbackDto
        :rtype: ~ignos.api.client.models.SchemaFeedbackDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measurement_form_instance_schema_feedback(
        self,
        id: str,
        schema_id: str,
        body: Optional[Union[_models.CreateMeasurementFormSchemaFeedbackRequest, IO]] = None,
        *,
        tenant_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SchemaFeedbackDto:
        """create_measurement_form_instance_schema_feedback.

        :param id: Required.
        :type id: str
        :param schema_id: Required.
        :type schema_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasurementFormSchemaFeedbackRequest or IO
        :keyword tenant_id: Default value is None.
        :paramtype tenant_id: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: SchemaFeedbackDto
        :rtype: ~ignos.api.client.models.SchemaFeedbackDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SchemaFeedbackDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasurementFormSchemaFeedbackRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_create_measurement_form_instance_schema_feedback_request(
            id=id,
            schema_id=schema_id,
            tenant_id=tenant_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SchemaFeedbackDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_supplier_measurement_form_instances(
        self, supplier_id: str, **kwargs: Any
    ) -> List[_models.MeasurementFormSupplierAccessInstanceDto]:
        """list_supplier_measurement_form_instances.

        :param supplier_id: Required.
        :type supplier_id: str
        :return: list of MeasurementFormSupplierAccessInstanceDto
        :rtype: list[~ignos.api.client.models.MeasurementFormSupplierAccessInstanceDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasurementFormSupplierAccessInstanceDto]]

        request = build_measurement_forms_instances_list_supplier_measurement_form_instances_request(
            supplier_id=supplier_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasurementFormSupplierAccessInstanceDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def upsert_supplier_to_measurment_form_instance(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        supplier_id: str,
        body: Optional[_models.UpsertSupplierToMeasurementFormInstanceRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """upsert_supplier_to_measurment_form_instance.

        :param id: Required.
        :type id: str
        :param supplier_id: Required.
        :type supplier_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpsertSupplierToMeasurementFormInstanceRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def upsert_supplier_to_measurment_form_instance(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        supplier_id: str,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """upsert_supplier_to_measurment_form_instance.

        :param id: Required.
        :type id: str
        :param supplier_id: Required.
        :type supplier_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def upsert_supplier_to_measurment_form_instance(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        supplier_id: str,
        body: Optional[Union[_models.UpsertSupplierToMeasurementFormInstanceRequest, IO]] = None,
        **kwargs: Any
    ) -> None:
        """upsert_supplier_to_measurment_form_instance.

        :param id: Required.
        :type id: str
        :param supplier_id: Required.
        :type supplier_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpsertSupplierToMeasurementFormInstanceRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpsertSupplierToMeasurementFormInstanceRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_upsert_supplier_to_measurment_form_instance_request(
            id=id,
            supplier_id=supplier_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def remove_supplier_from_measurment_form_instance(  # pylint: disable=inconsistent-return-statements
        self, id: str, supplier_id: str, **kwargs: Any
    ) -> None:
        """remove_supplier_from_measurment_form_instance.

        :param id: Required.
        :type id: str
        :param supplier_id: Required.
        :type supplier_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_forms_instances_remove_supplier_from_measurment_form_instance_request(
            id=id,
            supplier_id=supplier_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def delete_all_measurement_schema_instances(  # pylint: disable=inconsistent-return-statements
        self, **kwargs: Any
    ) -> None:
        """delete_all_measurement_schema_instances.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measurement_forms_instances_delete_all_measurement_schema_instances_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def export_dimension_report(
        self,
        id: str,
        body: Optional[_models.ExportDimensionReportRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.DownloadDto:
        """export_dimension_report.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.ExportDimensionReportRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def export_dimension_report(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.DownloadDto:
        """export_dimension_report.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def export_dimension_report(
        self, id: str, body: Optional[Union[_models.ExportDimensionReportRequest, IO]] = None, **kwargs: Any
    ) -> _models.DownloadDto:
        """export_dimension_report.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.ExportDimensionReportRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.DownloadDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "ExportDimensionReportRequest")
            else:
                _json = None

        request = build_measurement_forms_instances_export_dimension_report_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("DownloadDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class MeasuringToolsOperations:  # pylint: disable=too-many-public-methods
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`measuring_tools` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_measuring_tools(
        self,
        *,
        page: Optional[int] = None,
        page_size: int = 100,
        tool_id: Optional[str] = None,
        tool_name: Optional[str] = None,
        type_id: Optional[str] = None,
        sub_type_id: Optional[str] = None,
        order_by: Optional[str] = None,
        sort_direction: Optional[str] = None,
        include_deprecated: bool = False,
        **kwargs: Any
    ) -> _models.MeasuringToolDtoPagedData:
        """list_measuring_tools.

        :keyword page: Default value is None.
        :paramtype page: int
        :keyword page_size: Default value is 100.
        :paramtype page_size: int
        :keyword tool_id: Default value is None.
        :paramtype tool_id: str
        :keyword tool_name: Default value is None.
        :paramtype tool_name: str
        :keyword type_id: Default value is None.
        :paramtype type_id: str
        :keyword sub_type_id: Default value is None.
        :paramtype sub_type_id: str
        :keyword order_by: Default value is None.
        :paramtype order_by: str
        :keyword sort_direction: Default value is None.
        :paramtype sort_direction: str
        :keyword include_deprecated: Default value is False.
        :paramtype include_deprecated: bool
        :return: MeasuringToolDtoPagedData
        :rtype: ~ignos.api.client.models.MeasuringToolDtoPagedData
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolDtoPagedData]

        request = build_measuring_tools_list_measuring_tools_request(
            page=page,
            page_size=page_size,
            tool_id=tool_id,
            tool_name=tool_name,
            type_id=type_id,
            sub_type_id=sub_type_id,
            order_by=order_by,
            sort_direction=sort_direction,
            include_deprecated=include_deprecated,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolDtoPagedData", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measuring_tool(
        self,
        body: Optional[_models.CreateMeasuringToolRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """create_measuring_tool.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measuring_tool(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """create_measuring_tool.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measuring_tool(
        self, body: Optional[Union[_models.CreateMeasuringToolRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """create_measuring_tool.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolDetailDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasuringToolRequest")
            else:
                _json = None

        request = build_measuring_tools_create_measuring_tool_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolDetailDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measuring_tool(
        self,
        id: str,
        body: Optional[_models.UpdateMeasuringToolRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """update_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measuring_tool(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """update_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measuring_tool(
        self, id: str, body: Optional[Union[_models.UpdateMeasuringToolRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """update_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolDetailDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasuringToolRequest")
            else:
                _json = None

        request = build_measuring_tools_update_measuring_tool_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolDetailDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_measuring_tool(self, id: str, **kwargs: Any) -> _models.MeasuringToolDetailDto:
        """get_measuring_tool.

        :param id: Required.
        :type id: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolDetailDto]

        request = build_measuring_tools_get_measuring_tool_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolDetailDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_calibrations(self, id: str, **kwargs: Any) -> List[_models.MeasuringToolCalibrationDto]:
        """list_calibrations.

        :param id: Required.
        :type id: str
        :return: list of MeasuringToolCalibrationDto
        :rtype: list[~ignos.api.client.models.MeasuringToolCalibrationDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasuringToolCalibrationDto]]

        request = build_measuring_tools_list_calibrations_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasuringToolCalibrationDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def register_calibration(
        self,
        id: str,
        body: Optional[_models.RegisterMeasuringToolCalibrationRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """register_calibration.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.RegisterMeasuringToolCalibrationRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def register_calibration(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """register_calibration.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def register_calibration(
        self, id: str, body: Optional[Union[_models.RegisterMeasuringToolCalibrationRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """register_calibration.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.RegisterMeasuringToolCalibrationRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolCalibrationDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "RegisterMeasuringToolCalibrationRequest")
            else:
                _json = None

        request = build_measuring_tools_register_calibration_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolCalibrationDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def attach_calibration_certificate(
        self,
        id: str,
        calibration_id: str,
        body: Optional[_models.AttachCalibrationCertificateRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """attach_calibration_certificate.

        :param id: Required.
        :type id: str
        :param calibration_id: Required.
        :type calibration_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.AttachCalibrationCertificateRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def attach_calibration_certificate(
        self,
        id: str,
        calibration_id: str,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """attach_calibration_certificate.

        :param id: Required.
        :type id: str
        :param calibration_id: Required.
        :type calibration_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def attach_calibration_certificate(
        self,
        id: str,
        calibration_id: str,
        body: Optional[Union[_models.AttachCalibrationCertificateRequest, IO]] = None,
        **kwargs: Any
    ) -> _models.MeasuringToolCalibrationDto:
        """attach_calibration_certificate.

        :param id: Required.
        :type id: str
        :param calibration_id: Required.
        :type calibration_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.AttachCalibrationCertificateRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolCalibrationDto
        :rtype: ~ignos.api.client.models.MeasuringToolCalibrationDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolCalibrationDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "AttachCalibrationCertificateRequest")
            else:
                _json = None

        request = build_measuring_tools_attach_calibration_certificate_request(
            id=id,
            calibration_id=calibration_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolCalibrationDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_calibration_certificate(  # pylint: disable=inconsistent-return-statements
        self, id: str, calibration_id: str, **kwargs: Any
    ) -> None:
        """delete_calibration_certificate.

        :param id: Required.
        :type id: str
        :param calibration_id: Required.
        :type calibration_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measuring_tools_delete_calibration_certificate_request(
            id=id,
            calibration_id=calibration_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_manufacturers(self, **kwargs: Any) -> List[_models.ManufacturerDto]:
        """list_manufacturers.

        :return: list of ManufacturerDto
        :rtype: list[~ignos.api.client.models.ManufacturerDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.ManufacturerDto]]

        request = build_measuring_tools_list_manufacturers_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[ManufacturerDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_manufacturer(
        self,
        body: Optional[_models.CreateMeasuringToolManufacturer] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ManufacturerDto:
        """create_manufacturer.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolManufacturer
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ManufacturerDto
        :rtype: ~ignos.api.client.models.ManufacturerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_manufacturer(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ManufacturerDto:
        """create_manufacturer.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: ManufacturerDto
        :rtype: ~ignos.api.client.models.ManufacturerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_manufacturer(
        self, body: Optional[Union[_models.CreateMeasuringToolManufacturer, IO]] = None, **kwargs: Any
    ) -> _models.ManufacturerDto:
        """create_manufacturer.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolManufacturer or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: ManufacturerDto
        :rtype: ~ignos.api.client.models.ManufacturerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ManufacturerDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasuringToolManufacturer")
            else:
                _json = None

        request = build_measuring_tools_create_manufacturer_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ManufacturerDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_measuring_tool_types(self, **kwargs: Any) -> List[_models.MeasuringToolTypeDto]:
        """list_measuring_tool_types.

        :return: list of MeasuringToolTypeDto
        :rtype: list[~ignos.api.client.models.MeasuringToolTypeDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasuringToolTypeDto]]

        request = build_measuring_tools_list_measuring_tool_types_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasuringToolTypeDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measuring_tool_type(
        self,
        body: Optional[_models.CreateMeasuringToolType] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """create_measuring_tool_type.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolType
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measuring_tool_type(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """create_measuring_tool_type.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measuring_tool_type(
        self, body: Optional[Union[_models.CreateMeasuringToolType, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """create_measuring_tool_type.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolType or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolTypeDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasuringToolType")
            else:
                _json = None

        request = build_measuring_tools_create_measuring_tool_type_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolTypeDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measuring_tool_type(
        self,
        id: str,
        body: Optional[_models.UpdateMeasuringToolTypeRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """update_measuring_tool_type.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolTypeRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measuring_tool_type(
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """update_measuring_tool_type.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measuring_tool_type(
        self, id: str, body: Optional[Union[_models.UpdateMeasuringToolTypeRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolTypeDto:
        """update_measuring_tool_type.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolTypeRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolTypeDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasuringToolTypeRequest")
            else:
                _json = None

        request = build_measuring_tools_update_measuring_tool_type_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolTypeDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measuring_tool_type(  # pylint: disable=inconsistent-return-statements
        self, id: str, **kwargs: Any
    ) -> None:
        """delete_measuring_tool_type.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measuring_tools_delete_measuring_tool_type_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_measuring_tool_sub_types(self, type_id: str, **kwargs: Any) -> List[_models.MeasuringToolSubTypeDto]:
        """list_measuring_tool_sub_types.

        :param type_id: Required.
        :type type_id: str
        :return: list of MeasuringToolSubTypeDto
        :rtype: list[~ignos.api.client.models.MeasuringToolSubTypeDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasuringToolSubTypeDto]]

        request = build_measuring_tools_list_measuring_tool_sub_types_request(
            type_id=type_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasuringToolSubTypeDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measuring_tool_sub_type(
        self,
        type_id: str,
        body: Optional[_models.CreateMeasuringToolSubTypeRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """create_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolSubTypeRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measuring_tool_sub_type(
        self, type_id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """create_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measuring_tool_sub_type(
        self, type_id: str, body: Optional[Union[_models.CreateMeasuringToolSubTypeRequest, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """create_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringToolSubTypeRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolSubTypeDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasuringToolSubTypeRequest")
            else:
                _json = None

        request = build_measuring_tools_create_measuring_tool_sub_type_request(
            type_id=type_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolSubTypeDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_measuring_tool_sub_type(
        self,
        type_id: str,
        id: str,
        body: Optional[_models.UpdateMeasuringToolSubTypeRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """update_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolSubTypeRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_measuring_tool_sub_type(
        self, type_id: str, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """update_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_measuring_tool_sub_type(
        self,
        type_id: str,
        id: str,
        body: Optional[Union[_models.UpdateMeasuringToolSubTypeRequest, IO]] = None,
        **kwargs: Any
    ) -> _models.MeasuringToolSubTypeDto:
        """update_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateMeasuringToolSubTypeRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolSubTypeDto
        :rtype: ~ignos.api.client.models.MeasuringToolSubTypeDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolSubTypeDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateMeasuringToolSubTypeRequest")
            else:
                _json = None

        request = build_measuring_tools_update_measuring_tool_sub_type_request(
            type_id=type_id,
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolSubTypeDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measuring_tool_sub_type(  # pylint: disable=inconsistent-return-statements
        self, type_id: str, id: str, **kwargs: Any
    ) -> None:
        """delete_measuring_tool_sub_type.

        :param type_id: Required.
        :type type_id: str
        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measuring_tools_delete_measuring_tool_sub_type_request(
            type_id=type_id,
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_measuring_units(self, **kwargs: Any) -> List[_models.MeasuringUnitDto]:
        """list_measuring_units.

        :return: list of MeasuringUnitDto
        :rtype: list[~ignos.api.client.models.MeasuringUnitDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.MeasuringUnitDto]]

        request = build_measuring_tools_list_measuring_units_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[MeasuringUnitDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_measuring_unit(
        self,
        body: Optional[_models.CreateMeasuringUnit] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringUnitDto:
        """create_measuring_unit.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringUnit
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringUnitDto
        :rtype: ~ignos.api.client.models.MeasuringUnitDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_measuring_unit(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringUnitDto:
        """create_measuring_unit.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringUnitDto
        :rtype: ~ignos.api.client.models.MeasuringUnitDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_measuring_unit(
        self, body: Optional[Union[_models.CreateMeasuringUnit, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringUnitDto:
        """create_measuring_unit.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateMeasuringUnit or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringUnitDto
        :rtype: ~ignos.api.client.models.MeasuringUnitDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringUnitDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateMeasuringUnit")
            else:
                _json = None

        request = build_measuring_tools_create_measuring_unit_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringUnitDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_measuring_unit(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_measuring_unit.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_measuring_tools_delete_measuring_unit_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def get_measuring_tool_whitelist(
        self,
        *,
        page: Optional[int] = None,
        page_size: int = 100,
        filter: Optional[str] = None,
        order_by: Optional[str] = None,
        sort_direction: Optional[str] = None,
        **kwargs: Any
    ) -> _models.MeasuringToolWhitelistDtoPagedData:
        """get_measuring_tool_whitelist.

        :keyword page: Default value is None.
        :paramtype page: int
        :keyword page_size: Default value is 100.
        :paramtype page_size: int
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword order_by: Default value is None.
        :paramtype order_by: str
        :keyword sort_direction: Default value is None.
        :paramtype sort_direction: str
        :return: MeasuringToolWhitelistDtoPagedData
        :rtype: ~ignos.api.client.models.MeasuringToolWhitelistDtoPagedData
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolWhitelistDtoPagedData]

        request = build_measuring_tools_get_measuring_tool_whitelist_request(
            page=page,
            page_size=page_size,
            filter=filter,
            order_by=order_by,
            sort_direction=sort_direction,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolWhitelistDtoPagedData", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def whitelist_measuring_tool(
        self,
        body: Optional[_models.WhitelistMeasuringTool] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolWhitelistDto:
        """whitelist_measuring_tool.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.WhitelistMeasuringTool
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolWhitelistDto
        :rtype: ~ignos.api.client.models.MeasuringToolWhitelistDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def whitelist_measuring_tool(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolWhitelistDto:
        """whitelist_measuring_tool.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolWhitelistDto
        :rtype: ~ignos.api.client.models.MeasuringToolWhitelistDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def whitelist_measuring_tool(
        self, body: Optional[Union[_models.WhitelistMeasuringTool, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolWhitelistDto:
        """whitelist_measuring_tool.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.WhitelistMeasuringTool or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolWhitelistDto
        :rtype: ~ignos.api.client.models.MeasuringToolWhitelistDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolWhitelistDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "WhitelistMeasuringTool")
            else:
                _json = None

        request = build_measuring_tools_whitelist_measuring_tool_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolWhitelistDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def update_whitelisted_measuring_tool(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.UpdateWhitelistedMeasuringTool] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """update_whitelisted_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.UpdateWhitelistedMeasuringTool
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_whitelisted_measuring_tool(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """update_whitelisted_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_whitelisted_measuring_tool(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.UpdateWhitelistedMeasuringTool, IO]] = None, **kwargs: Any
    ) -> None:
        """update_whitelisted_measuring_tool.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.UpdateWhitelistedMeasuringTool or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "UpdateWhitelistedMeasuringTool")
            else:
                _json = None

        request = build_measuring_tools_update_whitelisted_measuring_tool_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def remove_measuring_tool_from_whitelist(self, id: str, **kwargs: Any) -> Optional[_models.ProblemDetails]:
        """remove_measuring_tool_from_whitelist.

        :param id: Required.
        :type id: str
        :return: ProblemDetails or None
        :rtype: ~ignos.api.client.models.ProblemDetails or None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[Optional[_models.ProblemDetails]]

        request = build_measuring_tools_remove_measuring_tool_from_whitelist_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204, 404]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = None
        if response.status_code == 404:
            deserialized = self._deserialize("ProblemDetails", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def list_unregistered_tool_values(self, **kwargs: Any) -> List[_models.UnregisteredToolValueDto]:
        """list_unregistered_tool_values.

        :return: list of UnregisteredToolValueDto
        :rtype: list[~ignos.api.client.models.UnregisteredToolValueDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.UnregisteredToolValueDto]]

        request = build_measuring_tools_list_unregistered_tool_values_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[UnregisteredToolValueDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def deprecate_tool(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        body: Optional[_models.DeprecateToolRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """deprecate_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.DeprecateToolRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def deprecate_tool(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """deprecate_tool.

        :param id: Required.
        :type id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def deprecate_tool(  # pylint: disable=inconsistent-return-statements
        self, id: str, body: Optional[Union[_models.DeprecateToolRequest, IO]] = None, **kwargs: Any
    ) -> None:
        """deprecate_tool.

        :param id: Required.
        :type id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.DeprecateToolRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "DeprecateToolRequest")
            else:
                _json = None

        request = build_measuring_tools_deprecate_tool_request(
            id=id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def get_calibration_list(
        self, *, status: Optional[Union[str, "_models.CalibrationListStatus"]] = None, **kwargs: Any
    ) -> List[_models.CalibrationListToolDto]:
        """get_calibration_list.

        :keyword status: Known values are: "SoonDue" and "Expired". Default value is None.
        :paramtype status: str or ~ignos.api.client.models.CalibrationListStatus
        :return: list of CalibrationListToolDto
        :rtype: list[~ignos.api.client.models.CalibrationListToolDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.CalibrationListToolDto]]

        request = build_measuring_tools_get_calibration_list_request(
            status=status,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[CalibrationListToolDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def import_measuring_tool(
        self,
        body: Optional[_models.ImportMeasuringTool] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """import_measuring_tool.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.ImportMeasuringTool
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def import_measuring_tool(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """import_measuring_tool.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def import_measuring_tool(
        self, body: Optional[Union[_models.ImportMeasuringTool, IO]] = None, **kwargs: Any
    ) -> _models.MeasuringToolDetailDto:
        """import_measuring_tool.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.ImportMeasuringTool or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: MeasuringToolDetailDto
        :rtype: ~ignos.api.client.models.MeasuringToolDetailDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.MeasuringToolDetailDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "ImportMeasuringTool")
            else:
                _json = None

        request = build_measuring_tools_import_measuring_tool_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("MeasuringToolDetailDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class PowerOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`power` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_power_regions(self, *, country: Optional[str] = None, **kwargs: Any) -> List[_models.PowerRegionDto]:
        """list_power_regions.

        :keyword country: Default value is None.
        :paramtype country: str
        :return: list of PowerRegionDto
        :rtype: list[~ignos.api.client.models.PowerRegionDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.PowerRegionDto]]

        request = build_power_list_power_regions_request(
            country=country,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[PowerRegionDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class PresentationOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`presentation` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get_component_settings(self, component_id: str, **kwargs: Any) -> _models.ComponentSettingsDto:
        """get_component_settings.

        :param component_id: Required.
        :type component_id: str
        :return: ComponentSettingsDto
        :rtype: ~ignos.api.client.models.ComponentSettingsDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ComponentSettingsDto]

        request = build_presentation_get_component_settings_request(
            component_id=component_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ComponentSettingsDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class SuppliersOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`suppliers` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_supplier_invites(self, **kwargs: Any) -> List[_models.SupplierInviteDto]:
        """list_supplier_invites.

        :return: list of SupplierInviteDto
        :rtype: list[~ignos.api.client.models.SupplierInviteDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.SupplierInviteDto]]

        request = build_suppliers_list_supplier_invites_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[SupplierInviteDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_supplier_invite(
        self,
        body: Optional[_models.CreateSupplierInvite] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SupplierInviteDto:
        """create_supplier_invite.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateSupplierInvite
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SupplierInviteDto
        :rtype: ~ignos.api.client.models.SupplierInviteDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_supplier_invite(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SupplierInviteDto:
        """create_supplier_invite.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: SupplierInviteDto
        :rtype: ~ignos.api.client.models.SupplierInviteDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_supplier_invite(
        self, body: Optional[Union[_models.CreateSupplierInvite, IO]] = None, **kwargs: Any
    ) -> _models.SupplierInviteDto:
        """create_supplier_invite.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateSupplierInvite or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: SupplierInviteDto
        :rtype: ~ignos.api.client.models.SupplierInviteDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.SupplierInviteDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateSupplierInvite")
            else:
                _json = None

        request = build_suppliers_create_supplier_invite_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("SupplierInviteDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_supplier_invite(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_supplier_invite.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_suppliers_delete_supplier_invite_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace
    def list_suppliers(self, **kwargs: Any) -> List[_models.ExternalSupplierDto]:
        """list_suppliers.

        :return: list of ExternalSupplierDto
        :rtype: list[~ignos.api.client.models.ExternalSupplierDto]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[List[_models.ExternalSupplierDto]]

        request = build_suppliers_list_suppliers_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("[ExternalSupplierDto]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def delete_supplier(self, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_supplier.

        :param id: Required.
        :type id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_suppliers_delete_supplier_request(
            id=id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})

    @overload
    def import_supplier(
        self, body: Optional[_models.ImportSupplier] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ExternalSupplierDto:
        """import_supplier.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.ImportSupplier
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ExternalSupplierDto
        :rtype: ~ignos.api.client.models.ExternalSupplierDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def import_supplier(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ExternalSupplierDto:
        """import_supplier.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: ExternalSupplierDto
        :rtype: ~ignos.api.client.models.ExternalSupplierDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def import_supplier(
        self, body: Optional[Union[_models.ImportSupplier, IO]] = None, **kwargs: Any
    ) -> _models.ExternalSupplierDto:
        """import_supplier.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.ImportSupplier or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: ExternalSupplierDto
        :rtype: ~ignos.api.client.models.ExternalSupplierDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ExternalSupplierDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "ImportSupplier")
            else:
                _json = None

        request = build_suppliers_import_supplier_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ExternalSupplierDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class SustainabilityOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`sustainability` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def test(self, **kwargs: Any) -> str:
        """test.

        :return: str
        :rtype: str
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[str]

        request = build_sustainability_test_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("str", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def get_consumption(
        self, body: Optional[_models.GetConsumption] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ConsumptionDto:
        """get_consumption.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.GetConsumption
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ConsumptionDto
        :rtype: ~ignos.api.client.models.ConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def get_consumption(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.ConsumptionDto:
        """get_consumption.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: ConsumptionDto
        :rtype: ~ignos.api.client.models.ConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def get_consumption(
        self, body: Optional[Union[_models.GetConsumption, IO]] = None, **kwargs: Any
    ) -> _models.ConsumptionDto:
        """get_consumption.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.GetConsumption or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: ConsumptionDto
        :rtype: ~ignos.api.client.models.ConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.ConsumptionDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "GetConsumption")
            else:
                _json = None

        request = build_sustainability_get_consumption_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("ConsumptionDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def get_consumption_with_automatic_granularity(
        self,
        body: Optional[_models.GetConsumptionAutoGranularity] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.GranluarConsumptionDto:
        """get_consumption_with_automatic_granularity.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.GetConsumptionAutoGranularity
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: GranluarConsumptionDto
        :rtype: ~ignos.api.client.models.GranluarConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def get_consumption_with_automatic_granularity(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.GranluarConsumptionDto:
        """get_consumption_with_automatic_granularity.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: GranluarConsumptionDto
        :rtype: ~ignos.api.client.models.GranluarConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def get_consumption_with_automatic_granularity(
        self, body: Optional[Union[_models.GetConsumptionAutoGranularity, IO]] = None, **kwargs: Any
    ) -> _models.GranluarConsumptionDto:
        """get_consumption_with_automatic_granularity.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.GetConsumptionAutoGranularity or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: GranluarConsumptionDto
        :rtype: ~ignos.api.client.models.GranluarConsumptionDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.GranluarConsumptionDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "GetConsumptionAutoGranularity")
            else:
                _json = None

        request = build_sustainability_get_consumption_with_automatic_granularity_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("GranluarConsumptionDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def get_power(
        self, body: Optional[_models.GetPower] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PowerDto:
        """get_power.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.GetPower
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: PowerDto
        :rtype: ~ignos.api.client.models.PowerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def get_power(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.PowerDto:
        """get_power.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: PowerDto
        :rtype: ~ignos.api.client.models.PowerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def get_power(self, body: Optional[Union[_models.GetPower, IO]] = None, **kwargs: Any) -> _models.PowerDto:
        """get_power.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.GetPower or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: PowerDto
        :rtype: ~ignos.api.client.models.PowerDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.PowerDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "GetPower")
            else:
                _json = None

        request = build_sustainability_get_power_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("PowerDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def get_customer_order_consumption(
        self,
        external_id: str,
        body: Optional[_models.CustomerOrderConsumptionRequest] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> JSON:
        """get_customer_order_consumption.

        :param external_id: Required.
        :type external_id: str
        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CustomerOrderConsumptionRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def get_customer_order_consumption(
        self, external_id: str, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> JSON:
        """get_customer_order_consumption.

        :param external_id: Required.
        :type external_id: str
        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def get_customer_order_consumption(
        self, external_id: str, body: Optional[Union[_models.CustomerOrderConsumptionRequest, IO]] = None, **kwargs: Any
    ) -> JSON:
        """get_customer_order_consumption.

        :param external_id: Required.
        :type external_id: str
        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CustomerOrderConsumptionRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[JSON]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CustomerOrderConsumptionRequest")
            else:
                _json = None

        request = build_sustainability_get_customer_order_consumption_request(
            external_id=external_id,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("object", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_work_order_consumption(self, external_id: str, **kwargs: Any) -> JSON:
        """get_work_order_consumption.

        :param external_id: Required.
        :type external_id: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[JSON]

        request = build_sustainability_get_work_order_consumption_request(
            external_id=external_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("object", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @distributed_trace
    def get_product_consumption(self, external_id: str, **kwargs: Any) -> JSON:
        """get_product_consumption.

        :param external_id: Required.
        :type external_id: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[JSON]

        request = build_sustainability_get_product_consumption_request(
            external_id=external_id,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("object", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    @overload
    def create_customer_order_line_ghg_report(
        self,
        body: Optional[_models.CreateCustomerOrderLineGhgReport] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.DownloadDto:
        """create_customer_order_line_ghg_report.

        :param body: Default value is None.
        :type body: ~ignos.api.client.models.CreateCustomerOrderLineGhgReport
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_customer_order_line_ghg_report(
        self, body: Optional[IO] = None, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.DownloadDto:
        """create_customer_order_line_ghg_report.

        :param body: Default value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Known values are: 'application/*+json', 'application/json', 'text/json'. Default value is
         "application/json".
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_customer_order_line_ghg_report(
        self, body: Optional[Union[_models.CreateCustomerOrderLineGhgReport, IO]] = None, **kwargs: Any
    ) -> _models.DownloadDto:
        """create_customer_order_line_ghg_report.

        :param body: Is either a model type or a IO type. Default value is None.
        :type body: ~ignos.api.client.models.CreateCustomerOrderLineGhgReport or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/*+json',
         'application/json', 'text/json'. Default value is None.
        :paramtype content_type: str
        :return: DownloadDto
        :rtype: ~ignos.api.client.models.DownloadDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.DownloadDto]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IO, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "CreateCustomerOrderLineGhgReport")
            else:
                _json = None

        request = build_sustainability_create_customer_order_line_ghg_report_request(
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("DownloadDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class UploadOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`upload` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def create_upload_info(self, **kwargs: Any) -> _models.UploadInfoDto:
        """create_upload_info.

        :return: UploadInfoDto
        :rtype: ~ignos.api.client.models.UploadInfoDto
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.UploadInfoDto]

        request = build_upload_create_upload_info_request(
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("UploadInfoDto", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized


class UsersOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`users` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_users(
        self,
        *,
        page_size: int = 50,
        filter: Optional[str] = None,
        continuation_token_parameter: Optional[str] = None,
        **kwargs: Any
    ) -> _models.UserDtoPagedResult:
        """list_users.

        :keyword page_size: Default value is 50.
        :paramtype page_size: int
        :keyword filter: Default value is None.
        :paramtype filter: str
        :keyword continuation_token_parameter: Default value is None.
        :paramtype continuation_token_parameter: str
        :return: UserDtoPagedResult
        :rtype: ~ignos.api.client.models.UserDtoPagedResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError}
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop("cls", None)  # type: ClsType[_models.UserDtoPagedResult]

        request = build_users_list_users_request(
            page_size=page_size,
            filter=filter,
            continuation_token_parameter=continuation_token_parameter,
            headers=_headers,
            params=_params,
        )
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize("UserDtoPagedResult", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
